import json
import datetime

from DIRAC.Core.Utilities import Time
from DIRAC.FrameworkSystem.Client.ComponentMonitoringClient import ComponentMonitoringClient
from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen, WErr


class ComponentHistoryHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getInstallationData(self):
        """
        Retrieves a list of dictionaries containing components to be displayed by the Component History page
        """
        # Get the selectors values
        req = self.__request()

        client = ComponentMonitoringClient()
        result = yield self.threadTask(
            client.getInstallations, req["installation"], req["component"], req["host"], True
        )
        if result["OK"]:
            values = []
            installations = result["Value"]
            for i in range(self.pageNumber, self.pageNumber + self.numberOfInstallations):
                if len(installations) > i:
                    installation = installations[i]
                else:
                    break
                uninstalled = "-"
                installedBy = "-"
                uninstalledBy = "-"
                if installation["UnInstallationTime"]:
                    uninstalled = installation["UnInstallationTime"].strftime("%Y-%m-%d %H:%M")
                if installation["InstalledBy"]:
                    installedBy = installation["InstalledBy"]
                if installation["UnInstalledBy"]:
                    uninstalledBy = installation["UnInstalledBy"]
                values.append(
                    {
                        "Name": installation["Instance"],
                        "Module": installation["Component"]["DIRACModule"],
                        "Host": installation["Host"]["HostName"],
                        "System": installation["Component"]["DIRACSystem"],
                        "Type": installation["Component"]["Type"],
                        "Installed": installation["InstallationTime"].strftime("%Y-%m-%d %H:%M"),
                        "Uninstalled": uninstalled,
                        "InstalledBy": installedBy,
                        "UninstalledBy": uninstalledBy,
                    }
                )
            timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
            total = len(installations)
            callback = {"success": "true", "result": values, "total": total, "date": timestamp}
        else:
            raise WErr.fromSERROR(result)
        self.finish(callback)

    @asyncGen
    def web_getSelectionData(self):
        """
        Returns a list of possible values for each different selector to choose from
        """

        req = self.__request()

        data = {}
        setup = self.getUserSetup().split("-")[-1]
        systemList = []
        system = None

        fields = ["name", "host", "module", "system", "type"]

        client = ComponentMonitoringClient()
        result = yield self.threadTask(client.getInstallations, {}, {}, {}, True)
        if result["OK"]:
            for field in fields:
                data[field] = set()
            for installation in result["Value"]:
                data["name"].add(installation["Instance"])
                data["host"].add(installation["Host"]["HostName"])
                data["module"].add(installation["Component"]["DIRACModule"])
                data["system"].add(installation["Component"]["DIRACSystem"])
                data["type"].add(installation["Component"]["Type"])

            # Order and format the results
            for field in fields:
                data[field] = list(data[field])
                data[field].sort()
                data[field] = [[x] for x in data[field]]
        else:
            raise WErr.fromSERROR(result)

        self.finish(data)

    def __request(self):
        """
        Returns a dictionary with the fields 'installation', 'component' and 'host'
        to be used by the getInstallations call in ComponentMonitoring service.
        The data inserted into the fields is retrieved from the values in the
        selectors
        """
        req = {"installation": {}, "component": {}, "host": {}}
        self.numberOfInstallations = int(self.get_argument("limit", "25"))
        self.pageNumber = int(self.get_argument("start", "0"))

        # Check every possible selector and get its value ( if any )
        if "name" in self.request.arguments:
            req["installation"]["Instance"] = list(json.loads(self.get_argument("name")))

        if "host" in self.request.arguments:
            req["host"]["HostName"] = list(json.loads(self.get_argument("host")))

        if "system" in self.request.arguments:
            req["component"]["System"] = list(json.loads(self.get_argument("system")))

        if "module" in self.request.arguments:
            req["component"]["Module"] = list(json.loads(self.get_argument("module")))

        if "type" in self.request.arguments:
            req["component"]["Type"] = list(json.loads(self.get_argument("type")))

        if "startDate" in self.request.arguments and len(self.get_argument("startDate")) > 0:
            if len(self.get_argument("startTime")) > 0:
                time = self.get_argument("startTime")
            else:
                time = "00:00"
            date = datetime.datetime.strptime("%s-%s" % (self.get_argument("startDate"), time), "%Y-%m-%d-%H:%M")
            req["installation"]["InstallationTime.bigger"] = date

        if "endDate" in self.request.arguments and len(self.get_argument("endDate")) > 0:
            if len(self.get_argument("endTime")) > 0:
                time = self.get_argument("endTime")
            else:
                time = "00:00"
            date = datetime.datetime.strptime("%s-%s" % (self.get_argument("endDate"), time), "%Y-%m-%d-%H:%M")
            req["installation"]["UnInstallationTime.smaller"] = date

        return req



import json

from diraccfg import CFG

from DIRAC import gConfig
from DIRAC.ConfigurationSystem.Client.ConfigurationClient import ConfigurationClient
from DIRAC.Core.Utilities import Time, List
from DIRAC.ConfigurationSystem.private.Modificator import Modificator

from WebAppDIRAC.Lib.WebHandler import WebSocketHandler, WErr, asyncGen


class ConfigurationManagerHandler(WebSocketHandler):

    AUTH_PROPS = "authenticated"

    def on_open(self):
        self.__configData = {}

    @asyncGen
    def on_message(self, msg):
        self.log.debug("RECEIVED", msg)
        try:
            params = json.loads(msg)
        except Exception:
            self.log.exception("No op defined")

        res = False
        if params["op"] == "forceRefresh":
            self.log.info("Initialize force refresh..")
            res = gConfig.forceRefresh(fromMaster=True)
        if params["op"] == "init":
            res = yield self.threadTask(self.__getRemoteConfiguration, "init")
        elif params["op"] == "getSubnodes":
            res = self.__getSubnodes(params["node"], params["nodePath"])
        elif params["op"] == "showConfigurationAsText":
            res = self.__showConfigurationAsText()
        elif params["op"] == "resetConfiguration":
            res = yield self.threadTask(self.__getRemoteConfiguration, "resetConfiguration")
        elif params["op"] == "getBulkExpandedNodeData":
            res = self.__getBulkExpandedNodeData(params["nodes"])
        elif params["op"] == "setOptionValue":
            res = self.__setOptionValue(params)
        elif params["op"] == "setComment":
            res = self.__setComment(params)
        elif params["op"] == "copyKey":
            res = self.__copyKey(params)
        elif params["op"] == "renameKey":
            res = self.__renameKey(params)
        elif params["op"] == "deleteKey":
            res = self.__deleteKey(params)
        elif params["op"] == "createSection":
            res = self.__createSection(params)
        elif params["op"] == "createOption":
            res = self.__createOption(params)
        elif params["op"] == "moveNode":
            res = self.__moveNode(params)
        elif params["op"] == "commitConfiguration":
            res = yield self.threadTask(self.__commitConfiguration)
        elif params["op"] == "showCurrentDiff":
            res = self.__showCurrentDiff()
        elif params["op"] == "showshowHistory":
            res = self.__history()
        elif params["op"] == "showDiff":
            res = self.__showDiff(params)
        elif params["op"] == "rollback":
            res = self.__rollback(params)
        elif params["op"] == "download":
            res = self.__download()
        elif params["op"] == "showCommitDiff":
            res = self.__showCommitDiff()

        self.log.debug("Sending back message", res)
        if res:
            self.write_message(res)

    def __getRemoteConfiguration(self, funcName):
        rpcClient = ConfigurationClient(
            url=gConfig.getValue("/DIRAC/Configuration/MasterServer", "Configuration/Server")
        )
        modCfg = Modificator(rpcClient)
        retVal = modCfg.loadFromRemote()

        if not retVal["OK"]:
            return {"success": 0, "op": "getSubnodes", "message": "The configuration cannot be read from the remote !"}

        self.__configData["cfgData"] = modCfg
        self.__configData["strCfgData"] = str(modCfg)

        version = str(modCfg.getCFG()["DIRAC"]["Configuration"]["Version"])  # pylint: disable=unsubscriptable-object
        configName = str(modCfg.getCFG()["DIRAC"]["Configuration"]["Name"])  # pylint: disable=unsubscriptable-object
        return {"success": 1, "op": funcName, "version": version, "name": configName}

    def __getSubnodes(self, parentNodeId, sectionPath):

        self.log.info("Expanding section", sectionPath)

        retData = []
        retVal = self.__getSubnodesForPath(sectionPath, retData)

        if not retVal:
            return {"success": 0, "op": "getSubnodes", "message": "Section %s does not exist" % sectionPath}

        return {"success": 1, "op": "getSubnodes", "nodes": retData, "parentNodeId": parentNodeId}

    def __getSubnodesForPath(self, sectionPath, retData):

        try:
            sectionCfg = self.__configData["cfgData"].getCFG()
            for section in [section for section in sectionPath.split("/") if not section.strip() == ""]:
                sectionCfg = sectionCfg[section]
        except Exception as v:
            self.log.exception("Section does not exist", "%s -> %s" % (sectionPath, repr(v)))
            return False

        for entryName in sectionCfg.listAll():
            comment = sectionCfg.getComment(entryName)
            nodeDef = {"text": entryName, "csName": entryName, "csComment": comment}
            nodeDef["leaf"] = False
            nodeDef["expanded"] = False
            if not sectionCfg.isSection(entryName):
                nodeDef["leaf"] = True
                if nodeDef.get("csName", "").lower() in ("password", "passwd", "passw"):
                    nodeDef["csValue"] = "******"
                else:
                    nodeDef["csValue"] = sectionCfg[entryName]

                nodeDef["text"] = nodeDef["text"] + " = " + nodeDef["csValue"]

            # Comment magic
            htmlC = self.__htmlComment(comment)
            if htmlC:
                qtipDict = {"text": htmlC}
                nodeDef["qtipCfg"] = qtipDict
            retData.append(nodeDef)

        return True

    def __htmlComment(self, rawComment):
        commentLines = []
        commiter = ""
        rawLines = rawComment.strip().split("\n")
        if rawLines[-1].find("@@-") == 0:
            commiter = rawLines[-1][3:]
            rawLines.pop(-1)
        for line in rawLines:
            line = line.strip()
            if not line:
                continue
            commentLines.append(line)
        if commentLines or commiter:
            return "%s<small><strong>%s</strong></small>" % ("<br/>".join(commentLines), commiter)
        else:
            return False

    def __showConfigurationAsText(self):
        # time.sleep(10)
        return {"success": 1, "op": "showConfigurationAsText", "text": self.__configData["strCfgData"]}

    def __getBulkExpandedNodeData(self, nodes):
        nodesPaths = nodes.split("<<||>>")
        returnData = []
        for nodePath in nodesPaths:
            pathData = []
            if self.__getSubnodesForPath(nodePath, pathData):
                returnData.append([nodePath, pathData])
        return {"success": 1, "op": "getBulkExpandedNodeData", "data": returnData}

    def __setOptionValue(self, params):
        try:
            optionPath = str(params["path"])
            optionValue = str(params["value"])
        except Exception as e:
            return {"success": 0, "op": "setOptionValue", "message": "Can't decode path or value: %s" % str(e)}

        self.__setCommiter()

        self.__configData["cfgData"].setOptionValue(optionPath, optionValue)

        if self.__configData["cfgData"].getValue(optionPath) == optionValue:
            self.log.info("Set option value", "%s = %s" % (optionPath, optionValue))
            return {"success": 1, "op": "setOptionValue", "parentNodeId": params["parentNodeId"], "value": optionValue}
        return {"success": 0, "op": "setOptionValue", "message": "Can't update %s" % optionPath}

    def __setComment(self, params):
        try:
            path = str(params["path"])
            value = str(params["value"])
        except Exception as e:
            return {"success": 0, "op": "setComment", "message": "Can't decode path or value: %s" % str(e)}

        self.__setCommiter()

        self.__configData["cfgData"].setComment(path, value)
        self.log.info("Set comment", "%s = %s" % (path, value))
        return {
            "success": 1,
            "op": "setComment",
            "parentNodeId": params["parentNodeId"],
            "comment": self.__configData["cfgData"].getComment(path),
        }

    def __renameKey(self, params):
        try:
            keyPath = str(params["path"]).strip()
            newName = str(params["newName"]).strip()
        except Exception as e:
            return {"success": 0, "op": "renameKey", "message": "Can't decode parameter: %s" % str(e)}
        try:
            if len(keyPath) == 0:
                return {"success": 0, "op": "renameKey", "message": "Entity path is not valid"}
            if len(newName) == 0:
                return {"success": 0, "op": "renameKey", "message": "Put any name for the entity!"}

            if self.__configData["cfgData"].existsOption(keyPath) or self.__configData["cfgData"].existsSection(
                keyPath
            ):
                self.__setCommiter()
                if self.__configData["cfgData"].renameKey(keyPath, newName):
                    return {"success": 1, "op": "renameKey", "parentNodeId": params["parentNodeId"], "newName": newName}
                else:
                    return {"success": 0, "op": "renameKey", "message": "There was a problem while renaming"}
            else:
                return {"success": 0, "op": "renameKey", "message": "Path doesn't exist"}
        except Exception as e:
            return {"success": 0, "op": "renameKey", "message": "Can't rename entity: %s" % str(e)}

    def __deleteKey(self, params):
        try:
            keyPath = str(params["path"]).strip()
        except Exception as e:
            return {"success": 0, "op": "deleteKey", "message": "Can't decode parameter: %s" % str(e)}
        try:
            if len(keyPath) == 0:
                return {"success": 0, "op": "deleteKey", "message": "Entity path is not valid"}
            if self.__configData["cfgData"].removeOption(keyPath) or self.__configData["cfgData"].removeSection(
                keyPath
            ):
                return {"success": 1, "op": "deleteKey", "parentNodeId": params["parentNodeId"]}
            else:
                return {"success": 0, "op": "deleteKey", "message": "Entity doesn't exist"}
        except Exception as e:
            return {"success": 0, "op": "deleteKey", "message": "Can't rename entity: %s" % str(e)}

    def __createSection(self, params):
        try:
            parentPath = str(params["path"]).strip()
            sectionName = str(params["name"]).strip()
            configText = str(params["config"]).strip()
        except Exception as e:
            return {"success": 0, "op": "createSection", "message": "Can't decode parameter: %s" % str(e)}
        try:
            if len(parentPath) == 0:
                return {"success": 0, "op": "createSection", "message": "Parent path is not valid"}
            if len(sectionName) == 0:
                return {"success": 0, "op": "createSection", "message": "Put any name for the section!"}
            sectionPath = "%s/%s" % (parentPath, sectionName)
            self.log.info("Creating section", sectionPath)

            self.__setCommiter()

            if self.__configData["cfgData"].createSection(sectionPath):
                nD = {
                    "text": sectionName,
                    "csName": sectionName,
                    "csComment": self.__configData["cfgData"].getComment(sectionPath),
                }
                htmlC = self.__htmlComment(nD["csComment"])
                if htmlC:
                    qtipDict = {"text": htmlC}
                    nD["qtipCfg"] = qtipDict
                #       If config Text is provided then a section is created out of that text
                if configText != "":
                    cfgData = self.__configData["cfgData"].getCFG()
                    newCFG = CFG()
                    newCFG.loadFromBuffer(configText)
                    self.__setCommiter()
                    self.__configData["cfgData"].mergeSectionFromCFG(sectionPath, newCFG)
                    return {
                        "success": 1,
                        "op": "createSection",
                        "parentNodeId": params["parentNodeId"],
                        "node": nD,
                        "sectionFromConfig": 1,
                    }
                else:
                    return {
                        "success": 1,
                        "op": "createSection",
                        "parentNodeId": params["parentNodeId"],
                        "node": nD,
                        "sectionFromConfig": 0,
                    }
            else:
                return {"success": 0, "op": "createSection", "message": "Section can't be created. It already exists?"}
        except Exception as e:
            return {"success": 0, "op": "createSection", "message": "Can't create section: %s" % str(e)}

    def __createOption(self, params):
        try:
            parentPath = str(params["path"]).strip()
            optionName = str(params["name"]).strip()
            optionValue = str(params["value"]).strip()
        except Exception as e:
            return {"success": 0, "op": "createOption", "message": "Can't decode parameter: %s" % str(e)}
        try:
            if len(parentPath) == 0:
                return {"success": 0, "op": "createOption", "message": "Parent path is not valid"}
            if len(optionName) == 0:
                return {"success": 0, "op": "createOption", "message": "Put any name for the option!"}
            if "/" in optionName:
                return {"success": 0, "op": "createOption", "message": "Options can't have a / in the name"}
            if len(optionValue) == 0:
                return {"success": 0, "op": "createOption", "message": "Options should have values!"}
            optionPath = "%s/%s" % (parentPath, optionName)
            self.log.info("Creating option", "%s = %s" % (optionPath, optionValue))
            if not self.__configData["cfgData"].existsOption(optionPath):
                self.__setCommiter()
                self.__configData["cfgData"].setOptionValue(optionPath, optionValue)

                return {
                    "success": 1,
                    "op": "createOption",
                    "parentNodeId": params["parentNodeId"],
                    "optionName": optionName,
                    "value": self.__configData["cfgData"].getValue(optionPath),
                    "comment": self.__configData["cfgData"].getComment(optionPath),
                }
            else:
                return {"success": 0, "op": "createOption", "message": "Option can't be created. It already exists?"}
        except Exception as e:
            return {"success": 0, "op": "createOption", "message": "Can't create option: %s" % str(e)}

    def __moveNode(self, params):
        try:
            nodePath = params["nodePath"]
            destinationParentPath = params["newParentPath"]
            beforeOfIndex = int(params["beforeOfIndex"])
        except Exception as e:
            return {
                "success": 0,
                "op": "moveNode",
                "message": "Can't decode parameter: %s" % str(e),
                "nodeId": params["nodeId"],
                "parentOldId": params["parentOldId"],
                "parentNewId": params["parentNewId"],
                "oldIndex": params["oldIndex"],
            }

        self.log.info(
            "Moving node", "Moving %s under %s before pos %s" % (nodePath, destinationParentPath, beforeOfIndex)
        )
        cfgData = self.__configData["cfgData"].getCFG()

        nodeDict = cfgData.getRecursive(nodePath)
        if not nodeDict:
            return {
                "success": 0,
                "op": "moveNode",
                "message": "Moving entity does not exist",
                "nodeId": params["nodeId"],
                "parentOldId": params["parentOldId"],
                "parentNewId": params["parentNewId"],
                "oldIndex": params["oldIndex"],
            }
        oldParentDict = cfgData.getRecursive(nodePath, -1)
        newParentDict = cfgData.getRecursive(destinationParentPath)
        if isinstance(newParentDict, str):
            return {
                "success": 0,
                "op": "moveNode",
                "message": "Destination is not a section",
                "nodeId": params["nodeId"],
                "parentOldId": params["parentOldId"],
                "parentNewId": params["parentNewId"],
                "oldIndex": params["oldIndex"],
            }
        if not newParentDict:
            return {
                "success": 0,
                "op": "moveNode",
                "message": "Destination does not exist",
                "nodeId": params["nodeId"],
                "parentOldId": params["parentOldId"],
                "parentNewId": params["parentNewId"],
                "oldIndex": params["oldIndex"],
            }
        # Calculate the old parent path
        oldParentPath = "/%s" % "/".join(List.fromChar(nodePath, "/")[:-1])
        if not oldParentPath == destinationParentPath and newParentDict["value"].existsKey(nodeDict["key"]):
            return {
                "success": 0,
                "op": "moveNode",
                "message": "Another entry with the same name already exists",
                "nodeId": params["nodeId"],
                "parentOldId": params["parentOldId"],
                "parentNewId": params["parentNewId"],
                "oldIndex": params["oldIndex"],
            }

        try:
            brothers = newParentDict["value"].listAll()
            if beforeOfIndex < len(brothers):
                nodeDict["beforeKey"] = brothers[beforeOfIndex]
            oldParentDict["value"].deleteKey(nodeDict["key"])
            addArgs = {}
            for key in ("comment", "beforeKey", "value", "key"):
                if key in nodeDict:
                    addArgs[key] = nodeDict[key]
            newParentDict["value"].addKey(**addArgs)
        except Exception as e:
            return {
                "success": 0,
                "op": "moveNode",
                "message": "Can't move node: %s" % str(e),
                "nodeId": params["nodeId"],
                "parentOldId": params["parentOldId"],
                "parentNewId": params["parentNewId"],
                "oldIndex": params["oldIndex"],
            }
        return {
            "success": 1,
            "op": "moveNode",
            "nodeId": params["nodeId"],
            "parentOldId": params["parentOldId"],
            "parentNewId": params["parentNewId"],
            "beforeOfIndex": params["beforeOfIndex"],
        }

    def __copyKey(self, params):
        try:
            nodePath = params["copyFromPath"]
            destinationParentPath = params["copyToPath"]
            newNodeName = params["newName"]
        except Exception as e:
            return {"success": 0, "op": "copyKey", "message": "Can't decode parameter: %s" % str(e)}

        #     gLogger.info( "Moving %s under %s before pos %s" % ( nodePath, destinationParentPath, beforeOfIndex ) )
        cfgData = self.__configData["cfgData"].getCFG()

        self.__setCommiter()

        nodeDict = cfgData.getRecursive(nodePath)

        if not nodeDict:
            return {"success": 0, "op": "copyKey", "message": "Moving entity does not exist"}
        newParentDict = cfgData.getRecursive(destinationParentPath)
        if isinstance(newParentDict, str):
            return {"success": 0, "op": "copyKey", "message": "Destination is not a section"}
        if not newParentDict:
            return {"success": 0, "op": "copyKey", "message": "Destination does not exist"}
        # Calculate the old parent path
        oldParentPath = "/%s" % "/".join(List.fromChar(nodePath, "/")[:-1])
        if not oldParentPath == destinationParentPath and newParentDict["value"].existsKey(newNodeName):
            return {"success": 0, "op": "copyKey", "message": "Another entry with the same name already exists"}

        try:
            nodeDict["key"] = newNodeName
            addArgs = {}
            for key in ("comment", "beforeKey", "key"):
                if key in nodeDict:
                    addArgs[key] = nodeDict[key]

            if isinstance(nodeDict["value"], str):
                newParentDict["value"].addKey(
                    nodeDict.get("key", ""),
                    nodeDict.get("value", ""),
                    nodeDict.get("comment", ""),
                    nodeDict.get("beforeKey", ""),
                )
            else:
                addArgs["value"] = nodeDict["value"].clone()
                newParentDict["value"].addKey(**addArgs)
        except Exception as e:
            return {"success": 0, "op": "copyKey", "message": "Can't move node: %s" % str(e)}
        return {
            "success": 1,
            "op": "copyKey",
            "newName": nodeDict["key"],
            "nodeId": params["nodeId"],
            "parentNodeToId": params["parentNodeToId"],
        }

    def __commitConfiguration(self):
        if not self.__authorizeAction():
            return {
                "success": 0,
                "op": "commitConfiguration",
                "message": "You are not authorized to commit configurations!",
            }
        self.log.always("User is commiting a new configuration version", "(%s)" % self.getUserDN())
        retDict = self.__configData["cfgData"].commit()
        if not retDict["OK"]:
            return {"success": 0, "op": "commitConfiguration", "message": retDict["Message"]}
        return {"success": 1, "op": "commitConfiguration"}

    def __authorizeAction(self):
        return "CSAdministrator" in self.getProperties()

    def __generateHTMLDiff(self, diffGen):
        diffList = []
        linesDiffList = []
        oldChange = False
        lineNumber = 0
        for diffLine in diffGen:
            if diffLine[0] == "-":
                diffList.append(("del", diffLine[1:], "", lineNumber))
                linesDiffList.append(["del", lineNumber])
                lineNumber = lineNumber + 1
            elif diffLine[0] == "+":
                if oldChange:
                    diffList[-1] = ("mod", diffList[-1][1], diffLine[1:], lineNumber)
                    linesDiffList[-1] = ["mod", lineNumber]
                    oldChange = False
                else:
                    diffList.append(("add", "", diffLine[1:], lineNumber))
                    linesDiffList.append(["add", lineNumber])
                    lineNumber = lineNumber + 1
            elif diffLine[0] == "?":
                if diffList[-1][0] == "del":
                    oldChange = True
                elif diffList[-1][0] == "mod":
                    diffList[-1] = ("conflict", diffList[-1][1], diffList[-1][2], lineNumber)
                    linesDiffList[-1] = ["conflict", lineNumber]
                elif diffList[-1][0] == "add":
                    diffList[-2] = ("mod", diffList[-2][1], diffList[-1][2], lineNumber)
                    linesDiffList[-2] = ["mod", lineNumber]
                    del diffList[-1]
                    lineNumber = lineNumber - 1
            else:
                diffList.append(("", diffLine[1:], diffLine[1:], lineNumber))
                lineNumber = lineNumber + 1

        return {"diff": diffList, "lines": linesDiffList, "totalLines": lineNumber}

    def __showCurrentDiff(self):
        if not self.__authorizeAction():
            return {
                "success": 0,
                "op": "showCurrentDiff",
                "message": "You are not authorized to commit configurations!",
            }
        diffGen = self.__configData["cfgData"].showCurrentDiff()
        processedData = self.__generateHTMLDiff(diffGen)
        return {
            "success": 1,
            "op": "showCurrentDiff",
            "lines": processedData["lines"],
            "totalLines": processedData["totalLines"],
            "html": self.render_string(
                "ConfigurationManager/diffConfig.tpl",
                titles=("Server's version", "User's current version"),
                diffList=processedData["diff"],
            ).decode("utf-8"),
        }

    def __showDiff(self, params):
        if not self.__authorizeAction():
            raise WErr(500, "You are not authorized to get diff's!")
        try:
            fromDate = str(params["fromVersion"])
            toDate = str(params["toVersion"])
        except Exception as e:
            raise WErr(500, "Can't decode params: %s" % e)
        rpcClient = ConfigurationClient(
            url=gConfig.getValue("/DIRAC/Configuration/MasterServer", "Configuration/Server")
        )
        modCfg = Modificator(rpcClient)
        diffGen = modCfg.getVersionDiff(fromDate, toDate)
        processedData = self.__generateHTMLDiff(diffGen)
        return {
            "success": 1,
            "op": "showDiff",
            "lines": processedData["lines"],
            "totalLines": processedData["totalLines"],
            "html": self.render_string(
                "ConfigurationManager/diffConfig.tpl",
                titles=("From version %s" % fromDate, "To version %s" % toDate),
                diffList=processedData["diff"],
            ).decode("utf-8"),
        }

    def __rollback(self, params):
        rollbackVersion = ""
        if not self.__authorizeAction():
            raise WErr(500, "You are not authorized to get diff's!")
        try:
            rollbackVersion = str(params["rollbackToVersion"])
        except Exception as e:
            raise WErr(500, "Can't decode params: %s" % e)
        rpcClient = ConfigurationClient(
            url=gConfig.getValue("/DIRAC/Configuration/MasterServer", "Configuration/Server")
        )
        modCfg = Modificator(rpcClient)
        retVal = modCfg.rollbackToVersion(rollbackVersion)
        if retVal["OK"]:
            return {"success": 1, "op": "rollback", "version": rollbackVersion}
        else:
            return {"success": 0, "op": "rollback", "message": retVal["Value"]}

    def __setCommiter(self):
        commiter = "%s@%s - %s" % (
            self.getUserName(),
            self.getUserGroup(),
            Time.dateTime().strftime("%Y-%m-%d %H:%M:%S"),
        )
        self.__configData["cfgData"].commiterId = commiter

    def __history(self):
        if not self.__authorizeAction():
            raise WErr(500, "You are not authorized to commit configurations!")
        rpcClient = ConfigurationClient(
            url=gConfig.getValue("/DIRAC/Configuration/MasterServer", "Configuration/Server")
        )
        retVal = rpcClient.getCommitHistory()
        if retVal["OK"]:
            cDict = {"numVersions": 0, "versions": []}
            for entry in retVal["Value"]:
                cDict["numVersions"] += 1
                cDict["versions"].append({"version": entry[1], "commiter": entry[0]})
        else:
            raise WErr.fromSERROR(retVal)
        return {"success": 1, "op": "showshowHistory", "result": cDict}

    def __download(self):

        version = str(self.__configData["cfgData"].getCFG()["DIRAC"]["Configuration"]["Version"])
        configName = str(self.__configData["cfgData"].getCFG()["DIRAC"]["Configuration"]["Name"])
        fileName = "cs.%s.%s" % (configName, version.replace(":", "").replace("-", "").replace(" ", ""))

        return {"success": 1, "op": "download", "result": self.__configData["strCfgData"], "fileName": fileName}

    def __showCommitDiff(self):
        """
        It returns only the modified CS content
        """

        if not self.__authorizeAction():
            return {
                "success": 0,
                "op": "showCurrentDiff",
                "message": "You are not authorized to commit configurations!! Bad boy!",
            }
        diffGen = self.__configData["cfgData"].showCurrentDiff()
        processedData = self.__generateHTMLDiff(diffGen)
        diffList = []
        allData = processedData.get("diff")
        for line in allData:
            if line[0] in ("conflict", "add", "mod", "del"):
                diffList += [line]
        return {
            "success": 1,
            "op": "showCommitDiff",
            "lines": processedData["lines"],
            "totalLines": processedData["totalLines"],
            "html": self.render_string(
                "ConfigurationManager/diffConfig.tpl",
                titles=("Server's version", "User's current version"),
                diffList=diffList,
            ).decode("utf-8"),
        }



import json

from DIRAC import gConfig, gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.Core.Utilities import DictCache
from DIRAC.Core.Utilities.Graphs.Palette import Palette
from DIRAC.RequestManagementSystem.Client.Request import Request
from DIRAC.RequestManagementSystem.Client.ReqClient import ReqClient
from DIRAC.WorkloadManagementSystem.Client.JobMonitoringClient import JobMonitoringClient
from DIRAC.WorkloadManagementSystem.Client.JobManagerClient import JobManagerClient
from DIRAC.WorkloadManagementSystem.Client.PilotManagerClient import PilotManagerClient
from DIRAC.WorkloadManagementSystem.Client.WMSAdministratorClient import WMSAdministratorClient
from DIRAC.WorkloadManagementSystem.Client.SandboxStoreClient import SandboxStoreClient

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen, WErr


class JobMonitorHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    __dataCache = DictCache.DictCache()

    @asyncGen
    def web_getJobData(self):
        req = self._request()

        result = yield self.threadTask(
            JobMonitoringClient().getJobPageSummaryWeb, req, self.globalSort, self.pageNumber, self.numberOfJobs
        )

        if not result["OK"]:
            self.finish({"success": "false", "result": [], "total": 0, "error": result["Message"]})
            return
        result = result["Value"]

        if "TotalRecords" not in result:
            self.finish(
                {
                    "success": "false",
                    "result": [],
                    "total": -1,
                    "error": "Data structure is corrupted",
                }
            )
            return

        if not result["TotalRecords"]:
            self.finish(
                {
                    "success": "false",
                    "result": [],
                    "total": 0,
                    "error": "There were no data matching your selection",
                }
            )
            return

        if "ParameterNames" not in result or "Records" not in result:
            self.finish(
                {
                    "success": "false",
                    "result": [],
                    "total": -1,
                    "error": "Data structure is corrupted",
                }
            )
            return

        if not result["ParameterNames"]:
            self.finish(
                {
                    "success": "false",
                    "result": [],
                    "total": -1,
                    "error": "ParameterNames field is missing",
                }
            )
            return

        if not result["Records"]:
            self.finish(
                {
                    "success": "false",
                    "result": [],
                    "total": 0,
                    "Message": "There are no data to display",
                }
            )
            return

        callback = []
        jobs = result["Records"]
        head = result["ParameterNames"]
        headLength = len(head)
        for i in jobs:
            tmp = {}
            for j in range(0, headLength):
                tmp[head[j]] = i[j]
            callback.append(tmp)
        total = result["TotalRecords"]
        extra = None
        if "Extras" in result:
            extra = result["Extras"]
            timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
            extra["date"] = timestamp

        callback = {"success": "true", "result": callback, "total": total, "extra": extra}
        self.finish(callback)

    def __dict2string(self, req):
        result = ""
        try:
            for key, value in req.iteritems():
                result = result + str(key) + ": " + ", ".join(value) + "; "
        except Exception as x:
            pass
            gLogger.info("\033[0;31m Exception: \033[0m %s" % x)
        result = result.strip()
        result = result[:-1]
        return result

    @asyncGen
    def web_getSelectionData(self):
        callback = {}
        user = self.getUserName()
        if not self.isRegisteredUser():
            callback["prod"] = [["Insufficient rights"]]
        else:
            cacheKey = (self.getUserGroup(), self.getUserSetup())

            callback = JobMonitorHandler.__dataCache.get(cacheKey)
            if not callback:
                callback = {}
                result = yield self.threadTask(JobMonitoringClient().getJobGroups)
                if result["OK"]:
                    prod = []
                    prods = result["Value"]
                    if prods:
                        prods.sort(reverse=True)
                        prod = [[i] for i in prods]
                    else:
                        prod = [["Nothing to display"]]
                else:
                    gLogger.error("JobMonitoringClient().getJobGroups() return error: %s" % result["Message"])
                    prod = [["Error happened on service side"]]
                callback["prod"] = prod

                result = yield self.threadTask(JobMonitoringClient().getSites)
                if result["OK"]:
                    tier1 = gConfig.getValue("/WebApp/PreferredSites", [])  # Always return a list
                    site = []
                    if result["Value"]:
                        s = list(result["Value"])
                        for i in tier1:
                            site.append([str(i)])
                        for i in s:
                            if i not in tier1:
                                site.append([str(i)])
                    else:
                        site = [["Nothing to display"]]
                else:
                    gLogger.error("JobMonitoringClient().getSites() return error: %s" % result["Message"])
                    site = [["Error happened on service side"]]
                callback["site"] = site
                # ##
                result = yield self.threadTask(JobMonitoringClient().getStates)
                if result["OK"]:
                    stat = []
                    if result["Value"]:
                        for i in result["Value"]:
                            stat.append([str(i)])
                    else:
                        stat = [["Nothing to display"]]
                else:
                    gLogger.error("JobMonitoringClient().getStates() return error: %s" % result["Message"])
                    stat = [["Error happened on service side"]]
                callback["status"] = stat
                # ##
                result = yield self.threadTask(JobMonitoringClient().getMinorStates)
                if result["OK"]:
                    stat = []
                    if result["Value"]:
                        for i in result["Value"]:
                            stat.append([i])
                    else:
                        stat = [["Nothing to display"]]
                else:
                    gLogger.error("JobMonitoringClient().getMinorStates() return error: %s" % result["Message"])
                    stat = [["Error happened on service side"]]
                callback["minorstat"] = stat
                # ##
                result = yield self.threadTask(JobMonitoringClient().getApplicationStates)
                if result["OK"]:
                    app = []
                    if result["Value"]:
                        for i in result["Value"]:
                            app.append([i])
                    else:
                        app = [["Nothing to display"]]
                else:
                    gLogger.error("JobMonitoringClient().getApplicationstates() return error: %s" % result["Message"])
                    app = [["Error happened on service side"]]
                callback["app"] = app
                # ##
                result = yield self.threadTask(JobMonitoringClient().getJobTypes)
                if result["OK"]:
                    types = []
                    if result["Value"]:
                        for i in result["Value"]:
                            types.append([i])
                    else:
                        types = [["Nothing to display"]]
                else:
                    gLogger.error("JobMonitoringClient().getJobTypes() return error: %s" % result["Message"])
                    types = [["Error happened on service side"]]
                callback["types"] = types
                # ##
                # groupProperty = credentials.getProperties(group)
                if not self.isRegisteredUser():
                    callback["owner"] = [["Insufficient rights"]]
                else:
                    result = yield self.threadTask(JobMonitoringClient().getOwners)
                    if result["OK"]:
                        owner = []
                        if result["Value"]:
                            for i in result["Value"]:
                                owner.append([str(i)])
                        else:
                            owner = [["Nothing to display"]]
                    elif "NormalUser" in self.getProperties():
                        owner = [[user]]
                        callback["owner"] = owner
                    else:
                        gLogger.error("JobMonitoringClient().getOwners() return error: %s" % result["Message"])
                        owner = [["Error happened on service side"]]
                    callback["owner"] = owner

                result = yield self.threadTask(JobMonitoringClient().getOwnerGroup)
                if result["OK"]:
                    callback["OwnerGroup"] = [[group] for group in result["Value"]]

                JobMonitorHandler.__dataCache.add(cacheKey, 360, callback)

        self.finish(callback)

    def _request(self):
        self.numberOfJobs = int(self.get_argument("limit", "25"))
        self.pageNumber = int(self.get_argument("start", "0"))
        self.globalSort = [["JobID", "DESC"]]
        req = {}

        jobids = list(json.loads(self.get_argument("JobID", "[]")))
        if jobids:
            req["JobID"] = jobids

        pilotids = list(json.loads(self.get_argument("PilotJobReference", "[]")))
        if pilotids:
            req["PilotJobReference"] = pilotids

        prodids = list(json.loads(self.get_argument("jobGroup", "[]")))
        if prodids:
            req["JobGroup"] = prodids

        sites = list(json.loads(self.get_argument("site", "[]")))
        if sites:
            req["Site"] = sites

        status = list(json.loads(self.get_argument("status", "[]")))
        if status:
            req["Status"] = status

        minorstat = list(json.loads(self.get_argument("minorStatus", "[]")))
        if minorstat:
            req["MinorStatus"] = minorstat

        apps = list(json.loads(self.get_argument("appStatus", "[]")))
        if apps:
            req["ApplicationStatus"] = apps

        types = list(json.loads(self.get_argument("jobType", "[]")))
        if types:
            req["JobType"] = types

        owner = list(json.loads(self.get_argument("owner", "[]")))
        if owner:
            req["Owner"] = owner

        ownerGroup = list(json.loads(self.get_argument("OwnerGroup", "[]")))
        if ownerGroup:
            req["OwnerGroup"] = ownerGroup

        if self.get_argument("startDate", ""):
            req["FromDate"] = self.get_argument("startDate")
            if self.get_argument("startTime", ""):
                req["FromDate"] += " " + self.get_argument("startTime")

        if self.get_argument("endDate", ""):
            req["ToDate"] = self.get_argument("endDate")
            if self.get_argument("endTime", ""):
                req["ToDate"] += " " + self.get_argument("endTime")

        if self.get_argument("date", ""):
            req["LastUpdate"] = self.get_argument("date")

        sort = json.loads(self.get_argument("sort", "[]"))
        if sort:
            self.globalSort = []
            for i in sort:
                if "LastSignOfLife" not in i["property"]:
                    self.globalSort += [[str(i["property"]), str(i["direction"])]]

        gLogger.debug("Request", str(req))
        return req

    @asyncGen
    def web_jobAction(self):
        ids = self.get_argument("JobID").split(",")
        ids = [int(i) for i in ids]

        RPC = JobManagerClient()
        if self.get_argument("action") == "delete":
            result = yield self.threadTask(RPC.deleteJob, ids)
        elif self.get_argument("action") == "kill":
            result = yield self.threadTask(RPC.killJob, ids)
        elif self.get_argument("action") == "reschedule":
            result = yield self.threadTask(RPC.rescheduleJob, ids)
        elif self.get_argument("action") == "reset":
            result = yield self.threadTask(RPC.resetJob, ids)

        callback = {}
        if result["OK"]:
            callback = {"success": "true", "result": ""}
        else:
            if "InvalidJobIDs" in result:
                callback = {"success": "false", "error": "Invalid JobIDs: %s" % result["InvalidJobIDs"]}
            elif "NonauthorizedJobIDs" in result:
                callback = {
                    "success": "false",
                    "error": "You are nonauthorized to %s jobs with JobID: %s"
                    % (self.get_argument("action"), result["NonauthorizedJobIDs"]),
                }
            else:
                callback = {"success": "false", "error": result["Message"]}
        self.finish(callback)

    @asyncGen
    def web_jobData(self):
        jobId = int(self.get_argument("id"))
        callback = {}

        if self.get_argument("data_kind") == "getJDL":
            RPC = JobMonitoringClient()
            result = yield self.threadTask(RPC.getJobJDL, jobId, False)
            if result["OK"]:
                callback = {"success": "true", "result": result["Value"]}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getBasicInfo":
            RPC = JobMonitoringClient()
            result = yield self.threadTask(RPC.getJobSummary, jobId)
            if result["OK"]:
                items = []
                for key, value in result["Value"].items():
                    items.append([key, value])
                callback = {"success": "true", "result": items}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getParams":
            RPC = JobMonitoringClient()
            result = yield self.threadTask(RPC.getJobParameters, jobId)
            if result["OK"]:
                attr = result["Value"].get(jobId, {})
                items = []
                for i in attr.items():
                    if i[0] == "Log URL":  # the link has to be opened in a new tab.
                        items.append([i[0], i[1].replace(">", ' target="_blank">')])
                    elif i[0] != "StandardOutput":
                        items.append([i[0], i[1]])

                callback = {"success": "true", "result": items}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getLoggingInfo":
            RPC = JobMonitoringClient()
            result = yield self.threadTask(RPC.getJobLoggingInfo, jobId)
            if result["OK"]:
                callback = {"success": "true", "result": result["Value"]}
            else:
                callback = {"success": "false", "error": result["Message"]}

        elif self.get_argument("data_kind") == "getStandardOutput":
            RPC = JobMonitoringClient()
            result = yield self.threadTask(RPC.getJobParameters, jobId)
            attr = result["Value"].get(jobId, {})
            if result["OK"]:
                if "StandardOutput" in attr:
                    callback = {"success": "true", "result": attr["StandardOutput"]}
                else:
                    callback = {"success": "false", "error": "Not accessible yet"}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getPending":

            result = yield self.threadTask(ReqClient().readRequestsForJobs, [jobId])

            if result["OK"]:
                items = {}
                if jobId in result["Value"]["Successful"]:
                    result = result["Value"]["Successful"][jobId].getDigest()
                    if result["OK"]:
                        items["PendingRequest"] = result["Value"]
                    else:
                        raise WErr.fromSERROR(result)
                    callback = {"success": "true", "result": items}
                elif jobId in result["Value"]["Failed"]:  # when no request associated to the job
                    callback = {"success": "false", "error": result["Value"]["Failed"][jobId]}
                else:
                    callback = {"success": "false", "error": "No request found with unknown reason"}
            else:
                callback = {"success": "false", "error": result["Message"]}

        elif self.get_argument("data_kind") == "getLogURL":
            RPC = JobMonitoringClient()
            result = yield self.threadTask(RPC.getJobParameters, jobId)
            if result["OK"]:
                attr = result["Value"].get(jobId, {})
                if "Log URL" in attr:
                    url = attr["Log URL"].split('"')
                    if "https:" not in url[1]:
                        # we can not open non secured URL
                        httpsUrl = url[1].replace("http", "https")
                    else:
                        httpsUrl = url[1]
                    callback = {"success": "true", "result": httpsUrl}
                else:
                    callback = {"success": "false", "error": "No URL found"}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getStagerReport":
            RPC = JobMonitoringClient()
            result = yield self.threadTask(RPC.getJobParameters, jobId)
            if result["OK"]:
                attr = result["Value"].get(jobId, {})
                if "StagerReport" in attr:
                    callback = {"success": "true", "result": attr["StagerReport"]}
                else:
                    callback = {"success": "false", "error": "StagerReport not available"}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getPilotStdOut":
            result = yield self.threadTask(WMSAdministratorClient().getJobPilotOutput, jobId)
            if result["OK"]:
                if "StdOut" in result["Value"]:
                    callback = {"success": "true", "result": result["Value"]["StdOut"]}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getPilotStdErr":
            result = yield self.threadTask(WMSAdministratorClient().getJobPilotOutput, jobId)
            if result["OK"]:
                if "StdErr" in result["Value"]:
                    callback = {"success": "true", "result": result["Value"]["StdErr"]}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getPilotLoggingInfo":
            retVal = yield self.threadTask(PilotManagerClient().getPilots, int(jobId))
            if retVal["OK"]:
                pilotReference = list(retVal["Value"])[0]
                retVal = yield self.threadTask(PilotManagerClient().getPilotLoggingInfo, pilotReference)
                if retVal["OK"]:
                    callback = {"success": "true", "result": retVal["Value"]}
                else:
                    callback = {"success": "false", "error": retVal["Message"]}
            else:
                callback = {"success": "false", "error": retVal["Message"]}
        self.finish(callback)

    @asyncGen
    def web_getStatisticsData(self):
        req = self._request()

        paletteColor = Palette()

        RPC = JobMonitoringClient()

        selector = self.get_argument("statsField")

        if selector == "Minor Status":
            selector = "MinorStatus"
        elif selector == "Application Status":
            selector = "ApplicationStatus"
        elif selector == "Job Group":
            selector = "JobGroup"
        elif selector == "Owner Group":
            selector = "OwnerGroup"
        elif selector == "Job Type":
            selector = "JobType"

        result = yield self.threadTask(RPC.getJobStats, selector, req)

        if result["OK"]:
            callback = []
            result = dict(result["Value"])
            keylist = sorted(result)
            if selector == "Site":
                tier1 = gConfig.getValue("/WebApp/PreferredSites", [])
                if tier1:
                    tier1.sort()
                    for i in tier1:
                        if i in result:
                            countryCode = i.rsplit(".", 1)[1]
                            callback.append(
                                {
                                    "key": i,
                                    "value": result[i],
                                    "code": countryCode,
                                    "color": paletteColor.getColor(countryCode),
                                }
                            )
            for key in keylist:
                if selector == "Site" and tier1:
                    if key not in tier1:
                        try:
                            countryCode = key.rsplit(".", 1)[1]
                        except BaseException:
                            countryCode = "Unknown"
                        callback.append(
                            {"key": key, "value": result[key], "code": countryCode, "color": paletteColor.getColor(key)}
                        )
                elif selector == "Site" and not tier1:
                    try:
                        countryCode = key.rsplit(".", 1)[1]
                    except BaseException:
                        countryCode = "Unknown"
                    callback.append(
                        {"key": key, "value": result[key], "code": countryCode, "color": paletteColor.getColor(key)}
                    )
                else:
                    callback.append({"key": key, "value": result[key], "code": "", "color": paletteColor.getColor(key)})
            callback = {"success": "true", "result": callback}
        else:
            callback = {"success": "false", "error": result["Message"]}
        self.finish(callback)

    @asyncGen
    def web_getSandbox(self):
        if "jobID" not in self.request.arguments:
            self.finish({"success": "false", "error": "Maybe you forgot the jobID ?"})
            return
        jobID = int(self.get_argument("jobID"))
        sbType = "Output"
        if "sandbox" in self.request.arguments:
            sbType = self.get_argument("sandbox")

        client = SandboxStoreClient(
            useCertificates=True,
            delegatedDN=self.getUserDN(),
            delegatedGroup=self.getUserGroup(),
            setup=self.getUserSetup(),
        )

        result = yield self.threadTask(client.downloadSandboxForJob, jobID, sbType, inMemory=True)

        if not result["OK"]:
            self.finish({"success": "false", "error": "Error: %s" % result["Message"]})
            return

        if "check" in self.request.arguments:
            self.finish({"success": "true"})
            return

        data = result["Value"]
        fname = "%s_%sSandbox.tar" % (str(jobID), sbType)
        self.set_header("Content-type", "application/x-tar")
        self.set_header("Content-Disposition", 'attachment; filename="%s"' % fname)
        self.set_header("Content-Length", len(data))
        self.set_header("Cache-Control", "no-cache, no-store, must-revalidate, max-age=0")
        self.set_header("Pragma", "no-cache")
        self.finish(data)



import json
from DIRAC import gConfig, gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.Core.Utilities.List import uniqueElements
from DIRAC.FrameworkSystem.Client.ProxyManagerClient import gProxyManager

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen, WErr


class ProxyManagerHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getSelectionData(self):
        callback = {}

        user = self.getUserName()
        if user.lower() == "anonymous":
            self.finish({"success": "false", "error": "You are not authorize to access these data"})

        if len(self.request.arguments) > 0:
            callback["extra"] = {i: self.get_argument(i) for i in self.request.arguments}
        result = yield self.threadTask(gProxyManager.getDBContents)
        if not result["OK"]:
            if result.get("Errno", 0) == 1112:
                raise WErr(503, "Connection error")
            self.finish({"success": "false", "error": result["Message"]})
        data = result["Value"]
        users = []
        groups = []
        for record in data["Records"]:
            users.append(str(record[0]))
            groups.append(str(record[2]))
        # AL:
        # for record in data["Dictionaries"]:
        #   users.append(record['user'])
        #   groups += record['groups']
        users = uniqueElements(users)
        groups = uniqueElements(groups)
        users.sort()
        groups.sort()
        users = [[x] for x in users]
        groups = [[x] for x in groups]

        callback["username"] = users
        callback["usergroup"] = groups
        result = gConfig.getOption("/WebApp/ProxyManagementMonitoring/TimeSpan", "86400,432000,604800,2592000")
        if result["OK"]:
            tmp = result["Value"]
            tmp = tmp.split(", ")
            if len(tmp) > 0:
                timespan = []
                for i in tmp:
                    human_readable = self.__humanize_time(i)
                    timespan.append([i, human_readable])
            else:
                timespan = [["Nothing to display"]]
        else:
            timespan = [["Error during RPC call"]]
        callback["expiredBefore"] = timespan
        callback["expiredAfter"] = timespan
        self.finish(callback)

    @asyncGen
    def web_getProxyManagerData(self):
        user = self.getUserName()
        if user.lower() == "anonymous":
            self.finish({"success": "false", "error": "You are not authorize to access these data"})
        start, limit, sort, req = self.__request()
        # pylint: disable=no-member
        result = yield self.threadTask(gProxyManager.getDBContents, req, sort, start, limit)
        # result = yield self.threadTask(gProxyManager.getDBContents, None, None, req, start, limit)
        gLogger.info("*!*!*!  RESULT: \n%s" % result)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
        svcData = result["Value"]
        proxies = []
        for record in svcData["Records"]:
            proxies.append(
                {
                    "proxyid": "%s@%s" % (record[1], record[2]),
                    "UserName": str(record[0]),
                    "UserDN": record[1],
                    "UserGroup": record[2],
                    "ExpirationTime": str(record[3]),
                    "PersistentFlag": str(record[4]),
                }
            )
        # for record in svcData['Dictionaries']:
        #   proxies.append({'proxyid': "%s@%s" % (record["DN"],
        #                                         record['groups'] if record['groups'] > 1 else record['groups'][0]),
        #                   'UserName': record['user'],
        #                   'UserDN': record['DN'],
        #                   'UserGroups': record['groups'],
        #                   'ExpirationTime': str(record['expirationtime']),
        #                   'Provider': record['provider']})
        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
        data = {"success": "true", "result": proxies, "total": svcData["TotalRecords"], "date": timestamp}
        self.finish(data)

    @asyncGen
    def web_deleteProxies(self):
        try:
            webIds = list(json.loads(self.get_argument("idList")))
        except Exception:
            self.finish({"success": "false", "error": "No valid id's specified"})
        idList = []
        for id in webIds:
            spl = id.split("@")
            dn = "@".join(spl[:-1])
            group = spl[-1]
            idList.append((dn, group))
        retVal = gProxyManager.deleteProxyBundle(idList)
        # for uid in webIds:
        #   spl = uid.split("@")
        #   dn = "@".join(spl[:-1])
        #   idList.append(dn)
        # retVal = yield self.threadTask(ProxyManagerClient().deleteProxy, idList)  # pylint: disable=no-member
        callback = {}
        if retVal["OK"]:
            callback = {"success": "true", "result": retVal["Value"]}
        else:
            callback = {"success": "false", "error": retVal["Message"]}
        self.finish(callback)

    def __humanize_time(self, sec=False):
        """
        Converts number of seconds to human readable values. Max return value is
        "More then a year" year and min value is "One day"
        """
        if not sec:
            return "Time span is not specified"
        try:
            sec = int(sec)
        except ValueError:
            return "Value from CS is not integer"

        month, week = divmod(sec, 2592000)
        if month > 12:
            return "More then a year"
        elif month > 1:
            return str(month) + " months"
        elif month == 1:
            return "One month"

        week, day = divmod(sec, 604800)
        if week == 1:
            return "One week"
        elif week > 1:
            return str(week) + " weeks"

        day, hours = divmod(sec, 86400)
        if day == 1:
            return "One day"
        elif day > 0:
            return str(day) + " days"

    def __request(self):
        gLogger.info("!!!  PARAMS: ", str(self.request.arguments))
        start = int(self.get_argument("start", "0"))
        limit = int(self.get_argument("limit", "25"))
        req = {}

        sortDirection = self.get_argument("sortDirection", "ASC")
        sortField = self.get_argument("sortField", "UserName")
        sort = [[sortField, sortDirection]]
        gLogger.info("!!!  S O R T : ", sort)

        users = list(json.loads(self.get_argument("username", "[]")))
        if users:
            req["UserName"] = users

        usersgroup = list(json.loads(self.get_argument("usergroup", "[]")))
        if usersgroup:
            req["UserGroup"] = usersgroup

        persistent = self.get_argument("persistent", "")
        if usersgroup and persistent in ["True", "False"]:
            req["PersistentFlag"] = persistent
        before = int(self.get_argument("expiredBefore", "0"))
        after = int(self.get_argument("expiredAfter", "0"))
        if before > after:
            before, after = after, before
        if before:
            req["beforeDate"] = before
        if after:
            req["afterDate"] = after
        gLogger.always("REQUEST:", req)
        return (start, limit, sort, req)



import json
from diraccfg import CFG

from WebAppDIRAC.Lib.WebHandler import WebSocketHandler, asyncGen
from DIRAC.ConfigurationSystem.Client.ConfigurationClient import ConfigurationClient
from DIRAC import gConfig, gLogger
from DIRAC.ConfigurationSystem.private.Modificator import Modificator


class RegistryManagerHandler(WebSocketHandler):

    AUTH_PROPS = "authenticated"

    def on_open(self):
        self.__configData = {}

    @asyncGen
    def on_message(self, msg):

        self.log.info("RECEIVED %s" % msg)
        try:
            params = json.loads(msg)
        except Exception:
            gLogger.exception("No op defined")

        res = False
        if params["op"] == "init":
            res = self.__getRemoteConfiguration("init")
        elif params["op"] == "getData":
            res = self.__getData(params)
        elif params["op"] == "deleteItem":
            res = self.__deleteItem(params)
        elif params["op"] == "addItem":
            res = self.__addItem(params)
        elif params["op"] == "editItem":
            res = self.__editItem(params)
        elif params["op"] == "resetConfiguration":
            res = self.__getRemoteConfiguration("resetConfiguration")
        elif params["op"] == "commitChanges":
            res = self.__commitChanges()
        elif params["op"] == "getGroupList":
            res = self.__getGroupList()
        elif params["op"] == "getRegistryProperties":
            res = self.__getRegistryProperties()
        elif params["op"] == "saveRegistryProperties":
            res = self.__saveRegistryProperties(params)
        elif params["op"] == "getVomsMapping":
            res = self.__getVomsMapping()
        elif params["op"] == "saveVomsMapping":
            res = self.__saveVomsMapping(params)

        if res:
            self.write_message(res)

    def __getRemoteConfiguration(self, funcName):
        rpcClient = ConfigurationClient(
            url=gConfig.getValue("/DIRAC/Configuration/MasterServer", "Configuration/Server")
        )
        modCfg = Modificator(rpcClient)
        retVal = modCfg.loadFromRemote()

        if not retVal["OK"]:
            return {"success": 0, "op": "getSubnodes", "message": "The configuration cannot be read from the remote !"}

        self.__configData["cfgData"] = modCfg
        self.__configData["strCfgData"] = str(modCfg)

        version = str(modCfg.getCFG()["DIRAC"]["Configuration"]["Version"])  # pylint: disable=unsubscriptable-object
        configName = str(modCfg.getCFG()["DIRAC"]["Configuration"]["Name"])  # pylint: disable=unsubscriptable-object
        return {"success": 1, "op": funcName, "version": version, "name": configName}

    def __getData(self, params):
        data = []
        if params["type"] == "users":

            sectionPath = "/Registry/Users"
            sectionCfg = self.getSectionCfg(sectionPath)

            for username in sectionCfg.listAll():

                item = {}
                item["name"] = username
                props = sectionCfg[username]

                item["dn"] = self.getIfExists("DN", props)
                item["ca"] = self.getIfExists("CA", props)
                item["email"] = self.getIfExists("Email", props)

                data.append(item)

        elif params["type"] == "groups":
            sectionPath = "/Registry/Groups"
            sectionCfg = self.getSectionCfg(sectionPath)

            for group in sectionCfg.listAll():
                item = {}
                item["name"] = group
                props = sectionCfg[group]

                item["users"] = self.getIfExists("Users", props)
                item["properties"] = self.getIfExists("Properties", props)
                item["vomsrole"] = self.getIfExists("VOMSRole", props)

                item["autouploadproxy"] = self.getIfExists("AutoUploadProxy", props)
                item["autouploadpilotproxy"] = self.getIfExists("AutoUploadPilotProxy", props)
                item["autoaddvoms"] = self.getIfExists("AutoAddVOMS", props)
                item["jobshare"] = self.getIfExists("JobShare", props)

                data.append(item)

        elif params["type"] == "hosts":
            sectionPath = "/Registry/Hosts"
            sectionCfg = self.getSectionCfg(sectionPath)

            for host in sectionCfg.listAll():
                item = {}
                item["name"] = host
                props = sectionCfg[host]

                item["dn"] = self.getIfExists("DN", props)
                item["properties"] = self.getIfExists("Properties", props)

                data.append(item)

        elif params["type"] == "voms":
            sectionPath = "/Registry/VOMS/Servers"
            sectionCfg = self.getSectionCfg(sectionPath)

            for host in sectionCfg.listAll():
                item = {}
                item["name"] = host

                data.append(item)

        elif params["type"] == "servers":
            sectionPath = "/Registry/VOMS/Servers/" + params["vom"]
            sectionCfg = self.getSectionCfg(sectionPath)

            for serv in sectionCfg.listAll():
                item = {}
                item["name"] = serv
                props = sectionCfg[serv]

                item["dn"] = self.getIfExists("DN", props)
                item["ca"] = self.getIfExists("CA", props)
                item["port"] = self.getIfExists("Port", props)

                data.append(item)

        return {"op": "getData", "success": 1, "type": params["type"], "data": data}

    def __getGroupList(self):
        data = []

        sectionPath = "/Registry/Groups"
        sectionCfg = self.getSectionCfg(sectionPath)

        for group in sectionCfg.listAll():
            data.append([group])

        return {"op": "getGroupList", "success": 1, "data": data}

    def __getVomsMapping(self):
        data = []

        sectionPath = "/Registry/VOMS/Mapping"
        sectionCfg = self.getSectionCfg(sectionPath)

        for mapping in sectionCfg.listAll():
            data.append({"name": mapping, "value": sectionCfg[mapping]})

        return {"op": "getVomsMapping", "success": 1, "data": data}

    def __getRegistryProperties(self):
        sectionPath = "/Registry"
        sectionCfg = self.getSectionCfg(sectionPath)

        data = {}
        for entryName in sectionCfg.listAll():
            if not sectionCfg.isSection(entryName):
                data[entryName] = sectionCfg[entryName]

        return {"op": "getRegistryProperties", "success": 1, "data": data}

    def getSectionCfg(self, sectionPath):
        sectionCfg = None
        try:
            sectionCfg = self.__configData["cfgData"].getCFG()
            for section in [section for section in sectionPath.split("/") if not section.strip() == ""]:
                sectionCfg = sectionCfg[section]
        except Exception as v:
            return False
        return sectionCfg

    def getIfExists(self, elem, propsList):
        if elem in propsList.listAll():
            return propsList[elem]
        else:
            return ""

    def __addItem(self, params):

        sectionPath = "/Registry/"
        configText = ""
        params = {k: str(v).strip() if v else v for k, v in params.items()}
        if params["type"] == "users":

            sectionPath = sectionPath + "Users"
            if params["dn"]:
                configText = "DN = " + params["dn"] + "\n"

            if params["ca"]:
                configText = configText + "CA = " + params["ca"] + "\n"

            if params["email"]:
                configText = configText + "Email = " + params["email"]

        elif params["type"] == "groups":

            sectionPath = sectionPath + "Groups"
            if params["users"]:
                configText = "Users = " + params["users"] + "\n"

            if params["properties"]:
                configText = configText + "Properties = " + params["properties"] + "\n"

            if params["jobshare"]:
                configText = configText + "JobShare = " + params["jobshare"] + "\n"

            if params["autouploadproxy"]:
                configText = configText + "AutoUploadProxy = " + params["autouploadproxy"] + "\n"

            if params["autouploadpilotproxy"]:
                configText = configText + "AutoUploadPilotProxy = " + params["autouploadpilotproxy"] + "\n"

            if params["autoaddvoms"]:
                configText = configText + "AutoAddVOMS = " + params["autoaddvoms"]

        elif params["type"] == "hosts":

            sectionPath = sectionPath + "Hosts"
            if params["dn"]:
                configText = "DN = " + params["dn"] + "\n"

            if params["properties"]:
                configText = configText + "Properties = " + params["properties"]

        elif params["type"] == "voms":

            sectionPath = sectionPath + "VOMS/Servers"

        elif params["type"] == "servers":

            sectionPath = sectionPath + "VOMS/Servers/" + params["vom"]
            if params["dn"]:
                configText = "DN = " + params["dn"] + "\n"

            if params["port"]:
                configText = configText + "Port = " + params["port"] + "\n"

            if params["ca"]:
                configText = configText + "CA = " + params["ca"]

        sectionPath = sectionPath + "/" + params["name"]

        if self.__configData["cfgData"].createSection(sectionPath):
            cfgData = self.__configData["cfgData"].getCFG()
            newCFG = CFG()
            newCFG.loadFromBuffer(configText)
            self.__configData["cfgData"].mergeSectionFromCFG(sectionPath, newCFG)
            return {"success": 1, "op": "addItem"}
        else:
            return {"success": 0, "op": "addItem", "message": "Section can't be created. It already exists?"}

    def __editItem(self, params):

        ret = self.__deleteItem(params)
        if ret["success"] == 1:
            ret = self.__addItem(params)
            ret["op"] = "editItem"
            return ret
        ret["op"] = "editItem"
        return ret

        sectionPath = "/Registry/"
        configText = ""
        params = {k: str(v).strip() if v else v for k, v in params.items()}
        if params["type"] == "users":

            sectionPath = sectionPath + "Users"
            if params["dn"]:
                configText = "DN = " + params["dn"] + "\n"

            if params["ca"]:
                configText = configText + "CA = " + params["ca"] + "\n"

            if params["email"]:
                configText = configText + "Email = " + params["email"]

        elif params["type"] == "groups":

            sectionPath = sectionPath + "Groups"
            if params["users"]:
                configText = "Users = " + params["users"] + "\n"

            if params["properties"]:
                configText = configText + "Properties = " + params["properties"] + "\n"

            if params["jobshare"]:
                configText = configText + "JobShare = " + params["jobshare"] + "\n"

            if params["autouploadproxy"]:
                configText = configText + "AutoUploadProxy = " + params["autouploadproxy"] + "\n"

            if params["autouploadpilotproxy"]:
                configText = configText + "AutoUploadPilotProxy = " + params["autouploadpilotproxy"] + "\n"

            if params["autoaddvoms"]:
                configText = configText + "AutoAddVOMS = " + params["autoaddvoms"]

        elif params["type"] == "hosts":

            sectionPath = sectionPath + "Hosts"
            if params["dn"]:
                configText = "DN = " + params["dn"] + "\n"

            if params["properties"]:
                configText = configText + "Properties = " + params["properties"]

        sectionPath = sectionPath + "/" + params["name"]

        #   deleting the options underneath
        sectionCfg = self.getSectionCfg(sectionPath)

        for opt in sectionCfg.listAll():
            self.__configData["cfgData"].removeOption(sectionPath + "/" + opt)

        newCFG = CFG()
        newCFG.loadFromBuffer(configText)
        self.__configData["cfgData"].mergeSectionFromCFG(sectionPath, newCFG)
        return {"success": 1, "op": "editItem"}

    def __deleteItem(self, params):
        sectionPath = "/Registry/"

        if params["type"] == "users":
            sectionPath = sectionPath + "Users"
        elif params["type"] == "groups":
            sectionPath = sectionPath + "Groups"
        elif params["type"] == "hosts":
            sectionPath = sectionPath + "Hosts"
        elif params["type"] == "voms":
            sectionPath = sectionPath + "VOMS/Servers"
        elif params["type"] == "servers":
            sectionPath = sectionPath + "VOMS/Servers/" + params["vom"]

        sectionPath = sectionPath + "/" + params["name"]
        if self.__configData["cfgData"].removeOption(sectionPath) or self.__configData["cfgData"].removeSection(
            sectionPath
        ):
            return {"success": 1, "op": "deleteItem"}
        else:
            return {"success": 0, "op": "deleteItem", "message": "Entity doesn't exist"}

    def __commitChanges(self):
        if "CSAdministrator" not in self.getProperties():
            return {"success": 0, "op": "commitChanges", "message": "You are not authorized to commit changes!!"}
        gLogger.always("User %s is commiting a new configuration version" % self.getUserDN())
        retDict = self.__configData["cfgData"].commit()
        if not retDict["OK"]:
            return {"success": 0, "op": "commitChanges", "message": retDict["Message"]}
        return {"success": 1, "op": "commitChanges"}

    def __saveRegistryProperties(self, params):
        sectionPath = "/Registry"
        sectionCfg = self.getSectionCfg(sectionPath)

        for opt in sectionCfg.listAll():
            if not sectionCfg.isSection(opt):
                self.__configData["cfgData"].removeOption(sectionPath + "/" + opt)

        configText = ""

        for newOpt in params:
            if newOpt != "op":
                if configText == "":
                    configText = newOpt + "=" + params[newOpt] + "\n"
                else:
                    configText = configText + newOpt + "=" + params[newOpt] + "\n"

        newCFG = CFG()
        newCFG.loadFromBuffer(configText)
        self.__configData["cfgData"].mergeSectionFromCFG(sectionPath, newCFG)

        return {"op": "saveRegistryProperties", "success": 1}

    def __saveVomsMapping(self, params):
        sectionPath = "/Registry/VOMS/Mapping"
        sectionCfg = self.getSectionCfg(sectionPath)

        for opt in sectionCfg.listAll():
            if not sectionCfg.isSection(opt):
                self.__configData["cfgData"].removeOption(sectionPath + "/" + opt)

        configText = ""

        for newOpt in params:
            if newOpt != "op":
                if configText == "":
                    configText = newOpt + "=" + params[newOpt] + "\n"
                else:
                    configText = configText + newOpt + "=" + params[newOpt] + "\n"

        newCFG = CFG()
        newCFG.loadFromBuffer(configText)
        self.__configData["cfgData"].mergeSectionFromCFG(sectionPath, newCFG)

        return {"op": "saveVomsMapping", "success": 1}



import ast
import json

from DIRAC.Core.Utilities import Time
from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen
from DIRAC.WorkloadManagementSystem.Client.VMClient import VMClient


class VMDiracHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getInstancesList(self):
        try:
            start = int(self.get_argument("start"))
        except Exception:
            start = 0
        try:
            limit = int(self.get_argument("limit"))
        except Exception:
            limit = 0

        sortField = "inst_InstanceID"
        sortDir = "DESC"
        if "sort" in self.request.arguments:
            sortValue = self.get_argument("sort")
            # converting the string into a dictionary
            sortValue = ast.literal_eval(sortValue.strip("[]"))
            sortField = str(sortValue["property"]).replace("_", ".")
            sortDir = str(sortValue["direction"])
        sort = [[sortField, sortDir]]

        condDict = {}
        if "cond" in self.request.arguments:
            dec = json.loads(self.get_argument("cond"))
            for k in dec:
                v = [str(dec[k])] if isinstance(dec[k], str) else [str(f) for f in dec[k]]
                condDict[str(k).replace("_", ".")] = v

        if "statusSelector" in self.request.arguments:
            condDict["inst.Status"] = [str(self.get_argument("statusSelector"))]

        result = VMClient().getInstancesContent(condDict, sort, start, limit)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        svcData = result["Value"]
        data = {"numRecords": svcData["TotalRecords"], "instances": []}
        for record in svcData["Records"]:
            rD = {}
            for iP in range(len(svcData["ParameterNames"])):
                param = svcData["ParameterNames"][iP].replace(".", "_")
                rD[param] = record[iP].strftime("%Y-%m-%d %H:%M:%S") if param == "inst_LastUpdate" else record[iP]
            data["instances"].append(rD)
        self.finish({"success": "true", "result": data["instances"], "total": data["numRecords"], "date": None})

    def web_stopInstances(self):
        webIds = json.loads(self.get_argument("idList"))
        result = VMClient().declareInstancesStopping(webIds)
        self.finish({"success": "true", "result": result})

    def web_getHistoryForInstance(self):
        instanceID = int(self.get_argument("instanceID"))
        result = VMClient().getHistoryForInstanceID(instanceID)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        svcData = result["Value"]
        data = []
        for record in svcData["Records"]:
            rD = {}
            for iP in range(len(svcData["ParameterNames"])):
                param = svcData["ParameterNames"][iP].replace(".", "_")
                rD[param] = record[iP].strftime("%Y-%m-%d %H:%M:%S") if param == "Update" else record[iP]
            data.append(rD)
        self.finish({"success": "true", "result": data, "total": len(data)})

    def web_checkVmWebOperation(self):
        operation = str(self.get_argument("operation"))
        result = VMClient().checkVmWebOperation(operation)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
        else:
            self.finish({"success": "true", "data": result["Value"]})

    def web_getHistoryValues(self):
        try:
            dbVars = [str(f) for f in json.loads(self.get_argument("vars"))]
        except Exception:
            dbVars = ["Load", "Jobs", "TransferredFiles"]
        try:
            timespan = int(self.get_argument("timespan"))
        except Exception:
            timespan = 86400
        result = VMClient().getHistoryValues(3600, {}, dbVars, timespan)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        svcData = result["Value"]
        data = []
        olderThan = Time.toEpoch() - 400
        for record in svcData["Records"]:
            rL = []
            for iP in range(len(svcData["ParameterNames"])):
                param = svcData["ParameterNames"][iP]
                rL.append(Time.toEpoch(record[iP]) if param == "Update" else record[iP])
            if rL[0] < olderThan:
                data.append(rL)
        self.finish({"success": "true", "data": data, "fields": svcData["ParameterNames"]})

    def web_getRunningInstancesHistory(self):
        try:
            bucketSize = int(self.get_argument("bucketSize"))
        except Exception:
            bucketSize = 900
        try:
            timespan = int(self.get_argument("timespan"))
        except Exception:
            timespan = 86400
        result = VMClient().getRunningInstancesHistory(timespan, bucketSize)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        svcData = result["Value"]
        data = []
        olderThan = Time.toEpoch() - 400
        rL = []
        for record in svcData:
            eTime = Time.toEpoch(record[0])
            if eTime < olderThan:
                rL = [eTime, int(record[1])]
            data.append(rL)
        self.finish({"success": "true", "data": data, "timespan": timespan})

    def web_getRunningInstancesBEPHistory(self):
        try:
            bucketSize = int(self.get_argument("bucketSize"))
        except Exception:
            bucketSize = 900
        try:
            timespan = int(self.get_argument("timespan"))
        except Exception:
            timespan = 86400
        result = VMClient().getRunningInstancesBEPHistory(timespan, bucketSize)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        svcData = result["Value"]
        data = []
        olderThan = Time.toEpoch() - 400
        for record in svcData:
            eTime = Time.toEpoch(record[0])
            if eTime < olderThan:
                rL = [eTime, record[1], int(record[2])]
            data.append(rL)
        self.finish({"success": "true", "data": data})

    def web_getRunningInstancesByRunningPodHistory(self):
        try:
            bucketSize = int(self.get_argument("bucketSize"))
        except Exception:
            bucketSize = 900
        try:
            timespan = int(self.get_argument("timespan"))
        except Exception:
            timespan = 86400
        result = VMClient().getRunningInstancesByRunningPodHistory(timespan, bucketSize)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        svcData = result["Value"]
        data = []
        olderThan = Time.toEpoch() - 400
        for record in svcData:
            eTime = Time.toEpoch(record[0])
            if eTime < olderThan:
                rL = [eTime, record[1], int(record[2])]
            data.append(rL)
        self.finish({"success": "true", "data": data})

    def web_getRunningInstancesByImageHistory(self):
        try:
            bucketSize = int(self.get_argument("bucketSize"))
        except Exception:
            bucketSize = 900
        try:
            timespan = int(self.get_argument("timespan"))
        except Exception:
            timespan = 86400
        result = VMClient().getRunningInstancesByImageHistory(timespan, bucketSize)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        svcData = result["Value"]
        data = []
        olderThan = Time.toEpoch() - 400
        for record in svcData:
            eTime = Time.toEpoch(record[0])
            if eTime < olderThan:
                rL = [eTime, record[1], int(record[2])]
            data.append(rL)
        self.finish({"success": "true", "data": data})



from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class PublicStateManagerHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getTreeMenuItems(self):
        applications = {}
        desktops = {}
        tree = {"text": ".", "children": [applications, desktops]}



""" WebApp handler for Downtimes WebApp
"""

import json
from datetime import datetime

import tornado
from DIRAC import gLogger
from DIRAC.ResourceStatusSystem.Client.PublisherClient import PublisherClient
from WebAppDIRAC.Lib.WebHandler import WebHandler, WErr, asyncGen


class DowntimesHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getSelectionData(self):
        callback = {"name": set(), "severity": set(), "sites": set()}

        gLogger.info("web_getSelectionData arguments", repr(self.request.arguments))

        downtimes = yield self.threadTask(PublisherClient().getCachedDowntimes, None, None, None, None)

        if downtimes["OK"]:

            dtList = [dict(zip(downtimes["Columns"], dt)) for dt in downtimes["Value"]]

            for dt in dtList:

                callback["name"].add(dt["Name"])
                callback["severity"].add(dt["Severity"])

        sites = yield self.threadTask(PublisherClient().getSites)
        if sites["OK"]:

            callback["site"] = sites["Value"]

        for key, value in callback.items():

            callback[key] = [[item] for item in list(value)]
            # callback[key].sort()
            callback[key] = [["All"]] + callback[key]

        callback["view"] = [["tabular"], ["availability"]]

        self.finish(callback)

    @asyncGen
    def web_getDowntimesData(self):
        requestParams = self.__requestParams()
        gLogger.info(requestParams)

        retVal = yield self.threadTask(PublisherClient().getSitesResources, list(requestParams["site"]))

        sitesResources = ""
        if not retVal["OK"]:
            raise WErr.fromSERROR(retVal)
        else:
            sitesResources = retVal["Value"]

        names = []
        if requestParams["site"]:

            if names is None:
                names = []

            for _site, resources in sitesResources.items():

                names += resources["ces"]
                names += resources["ses"]

        downtimes = PublisherClient().getCachedDowntimes(None, None, names, list(requestParams["severity"]))
        if not downtimes["OK"]:
            raise WErr.fromSERROR(downtimes)

        dtList = [dict(zip(downtimes["Columns"], dt)) for dt in downtimes["Value"]]

        for dt in dtList:

            dt["Site"] = "Unknown"

            for site, resources in sitesResources.items():
                if dt["Name"] in resources["ces"] + resources["ses"]:
                    dt["Site"] = site
                    break

            dt["StartDate"] = str(dt["StartDate"])
            dt["EndDate"] = str(dt["EndDate"])

        self.finish({"success": "true", "result": dtList, "total": len(dtList)})

    def __requestParams(self):
        """
        We receive the request and we parse it, in this case, we are doing nothing,
        but it can be certainly more complex.
        """
        gLogger.always("!!!  PARAMS: ", repr(self.request.arguments))

        responseParams = {"name": [], "severity": [], "site": []}

        for key in responseParams:
            if self.get_argument(key, None):
                responseParams[key] = list(json.loads(self.get_argument(key)))

        try:
            startDate = "%s %s" % (self.get_argument("startDate"), self.get_argument("startTime"))
            startDate = datetime.strptime(startDate, "%Y-%m-%d %H:%M")
        except (tornado.web.MissingArgumentError, ValueError):
            startDate = datetime.utcnow()
        responseParams["startDate"] = startDate

        try:
            endDate = "%s %s" % (self.get_argument("endDate"), self.get_argument("endTime"))
            endDate = datetime.strptime(endDate, "%Y-%m-%d %H:%M")
        except (tornado.web.MissingArgumentError, ValueError):
            endDate = datetime.utcnow()
        responseParams["endDate"] = endDate

        return responseParams



""" Get Site Summary page content
"""

import json

from DIRAC import gLogger
from DIRAC.Core.Utilities.SiteSEMapping import getSEsForSite
from DIRAC.Core.Utilities.Plotting.FileCoding import codeRequestInFileId
from DIRAC.ConfigurationSystem.Client.Helpers.Resources import getSiteCEMapping, getGOCSiteName, getDIRACSiteName
from DIRAC.ResourceStatusSystem.Client.PublisherClient import PublisherClient

from WebAppDIRAC.Lib.WebHandler import asyncGen
from WebAppDIRAC.WebApp.handler.ResourceSummaryHandler import SummaryHandlerMix


class SiteSummaryHandler(SummaryHandlerMix):

    ELEMENT_TYPE = "Site"

    @asyncGen
    def web_getSelectionData(self):
        callback = yield self.threadTask(self._getSelectionData)
        self.finish(callback)

    @asyncGen
    def web_expand(self):
        callback = yield self.threadTask(self._expand)
        self.finish(callback)

    @asyncGen
    def web_action(self):
        callback = yield self.threadTask(self._action)
        self.finish(callback)

    @asyncGen
    def web_getSiteSummaryData(self):
        """This method returns the data required to fill the grid."""
        requestParams = self.__requestParams()
        gLogger.info(requestParams)

        pub = PublisherClient()

        elementStatuses = yield self.threadTask(
            pub.getElementStatuses,
            "Site",
            requestParams["name"],
            requestParams["elementType"],
            requestParams["statusType"],
            requestParams["status"],
            requestParams["tokenOwner"],
        )
        if not elementStatuses["OK"]:
            self.finish({"success": "false", "error": elementStatuses["Message"]})
            return

        elementList = [dict(zip(elementStatuses["Columns"], site)) for site in elementStatuses["Value"]]

        for elementStatus in elementList:

            elementStatus["Country"] = elementStatus["Name"][-2:]
            elementStatus["DateEffective"] = str(elementStatus["DateEffective"])
            elementStatus["LastCheckTime"] = str(elementStatus["LastCheckTime"])
            elementStatus["TokenExpiration"] = str(elementStatus["TokenExpiration"])

        result = {"success": "true", "result": elementList, "total": len(elementList)}

        self.finish(result)

    def _getInfo(self, requestParams):

        gLogger.info(requestParams)

        if not requestParams["name"]:
            gLogger.warn("No name given")
            self.finish({"success": "false", "error": "We need a Site Name to generate an Overview"})
            return

        elementName = requestParams["name"][0]

        pub = PublisherClient()

        elementStatuses = yield self.threadTask(
            pub.getElementStatuses, "Site", str(elementName), None, "all", None, None
        )

        if not elementStatuses["OK"]:
            gLogger.error(elementStatuses["Message"])
            self.finish({"success": "false", "error": "Error getting ElementStatus information"})
            return

        if not elementStatuses["Value"]:
            gLogger.error('element "%s" not found' % elementName)
            self.finish({"success": "false", "error": 'element "%s" not found' % elementName})
            return

        elementStatus = [dict(zip(elementStatuses["Columns"], element)) for element in elementStatuses["Value"]][0]
        elementStatus["DateEffective"] = str(elementStatus["DateEffective"])
        elementStatus["LastCheckTime"] = str(elementStatus["LastCheckTime"])
        elementStatus["TokenExpiration"] = str(elementStatus["TokenExpiration"])

        gocdb_name = getGOCSiteName(elementName)
        if not gocdb_name["OK"]:
            gLogger.error(gocdb_name["Message"])
            elementStatus["GOCDB"] = ""
            gocdb_name = ""
        else:
            gocdb_name = gocdb_name["Value"]
            elementStatus["GOCDB"] = '<a href="https://goc.egi.eu/portal/index.php?Page_'
            elementStatus["GOCDB"] += 'Type=Submit_Search&SearchString=%s" target="_blank">%s</a>' % (
                gocdb_name,
                gocdb_name,
            )

        dirac_names = getDIRACSiteName(gocdb_name)
        if not dirac_names["OK"]:
            gLogger.error(dirac_names["Message"])
            dirac_names = []
        else:
            elementStatus["GOCDB"] += "("
            for i in dirac_names["Value"]:
                elementStatus["GOCDB"] += "%s " % i
            elementStatus["GOCDB"] += ")"

        elementStatus["GGUS"] = '<a href="https://ggus.eu/ws/ticket_search.php?'
        elementStatus["GGUS"] += "show_columns_check[]=REQUEST_ID&"
        elementStatus["GGUS"] += "show_columns_check[]=TICKET_TYPE&show_columns_check[]=AFFECTED_VO&"
        elementStatus["GGUS"] += "show_columns_check[]=AFFECTED_SITE&show_columns_check[]=PRIORITY"
        elementStatus["GGUS"] += "&show_columns_check[]=RESPONSIBLE_UNIT&"
        elementStatus["GGUS"] += "show_columns_check[]=STATUS&show_columns_check[]=DATE_OF_CREATION&"
        elementStatus["GGUS"] += "show_columns_check[]=LAST_UPDATE&show_columns_check[]=TYPE_OF_PROBLEM&"
        elementStatus["GGUS"] += "show_columns_check[]=SUBJECT&ticket=&supportunit=all&su_hierarchy=all&"
        elementStatus["GGUS"] += "vo=all&user=&keyword=&involvedsupporter=&assignto=&"
        elementStatus["GGUS"] += "affectedsite=%s&specattrib=0&status=open&priority=all&" % gocdb_name
        elementStatus["GGUS"] += "typeofproblem=all&ticketcategory=&mouarea=&technology_provider=&"
        elementStatus["GGUS"] += "date_type=creation+date&radiotf=1&timeframe=any&from_date=&to_date=&"
        elementStatus["GGUS"] += "untouched_date=&orderticketsby=GHD_INT_REQUEST_ID&"
        elementStatus["GGUS"] += 'orderhow=descending" target="_blank"> %s tickets</a>' % gocdb_name

        convertName = {
            "CERN-PROD": "CERN",
            "INFN-T1": "CNAF",
            "FZK-LCG2": "GridKa",
            "IN2P3-CC": "IN2P3",
            "NIKHEF-ELPROD": "NIKHEF",
            "pic": "PIC",
            "RAL-LCG2": "RAL",
            "SARA-MATRIX": "SARA",
        }

        elog = convertName.get(gocdb_name, "")

        elementStatus["Elog"] = (
            '<a href="https://lblogbook.cern.ch/Operations/?Site=^'
            + elog
            + '%24&mode=summary" target="_blank">'
            + elog
            + "</a>"
        )

        self.finish({"success": "true", "result": elementStatus, "total": len(elementStatus)})

    def _getStorages(self, requestParams):

        if not requestParams["name"]:
            gLogger.warn("No name given")
            self.finish({"success": "false", "error": "We need a Site Name to generate an Overview"})
            return

        pub = PublisherClient()

        elementName = requestParams["name"][0]
        retVal = getSEsForSite(elementName)
        if retVal["OK"]:
            storageElements = retVal["Value"]
        else:
            self.finish({"success": "false", "error": retVal["Message"]})
            return

        storageElementsStatus = []
        gLogger.info("storageElements = " + str(storageElements))

        # FIXME: use properly RSS
        for se in storageElements:
            sestatuses = yield self.threadTask(pub.getElementStatuses, "Resource", se, None, None, None, None)

            for sestatus in sestatuses["Value"]:
                storageElementsStatus.append([sestatus[0], sestatus[1], sestatus[2], sestatus[6]])

        self.finish({"success": "true", "result": storageElementsStatus, "total": len(storageElementsStatus)})

    def _getComputingElements(self, requestParams):

        if not requestParams["name"]:
            gLogger.warn("No name given")
            self.finish({"success": "false", "error": "We need a Site Name to generate an Overview"})
            return

        pub = PublisherClient()

        elementName = requestParams["name"][0]

        res = getSiteCEMapping()
        if not res["OK"]:
            self.finish({"success": "false", "error": res["Message"]})
            return
        computing_elements = res["Value"][elementName]
        computing_elements_status = []
        gLogger.info("computing_elements = " + str(computing_elements))

        for ce in computing_elements:
            cestatuses = yield self.threadTask(pub.getElementStatuses, "Resource", ce, None, "all", None, None)
            gLogger.info("cestatus = " + str(cestatuses))

            for cestatus in cestatuses["Value"]:
                computing_elements_status.append([cestatus[0], cestatus[1], cestatus[2], cestatus[6]])

        self.finish({"success": "true", "result": computing_elements_status, "total": len(computing_elements_status)})

    def _getImages(self, requestParams):

        if not requestParams["name"]:
            gLogger.warn("No name given")
            self.finish({"success": "false", "error": "We need a Site Name to generate an Overview"})
            return

        elementName = requestParams["name"][0]
        pub = PublisherClient()

        elementStatuses = yield self.threadTask(
            pub.getElementStatuses, "Site", str(elementName), None, "all", None, None
        )

        if not elementStatuses["Value"]:
            gLogger.error('element "%s" not found' % elementName)
            self.finish({"success": "false", "error": 'element "%s" not found' % elementName})
            return

        elementStatus = [dict(zip(elementStatuses["Columns"], element)) for element in elementStatuses["Value"]][0]

        plotDict1 = self.getPlotDict(
            elementStatus["Name"], "FinalMajorStatus", "RunningJobs", "Job", plotTitle="Final Minor Status of jobs"
        )
        image1 = codeRequestInFileId(plotDict1)["Value"]["plot"]

        plotDict2 = self.getPlotDict(elementStatus["Name"], "GridStatus", "NumberOfPilots", "Pilot")
        image2 = codeRequestInFileId(plotDict2)["Value"]["plot"]

        plotDict3 = self.getPlotDict(
            elementStatus["Name"], "JobType", "Job execution rate", "Job", plotTitle="Jobs execution rate by job type"
        )
        image3 = codeRequestInFileId(plotDict3)["Value"]["plot"]

        plotDict4 = self.getPlotDict(
            elementStatus["Name"], "JobSplitType", "NumberOfJobs", "WMSHistory", status="Running"
        )
        image4 = codeRequestInFileId(plotDict4)["Value"]["plot"]

        plotDict5 = self.getPlotDict(elementStatus["Name"], "Channel", "SuceededTransfers", "DataOperation")
        image5 = codeRequestInFileId(plotDict5)["Value"]["plot"]

        plotDict6 = self.getPlotDict(elementStatus["Name"], "FinalStatus", "FailedTransfers", "DataOperation")
        image6 = codeRequestInFileId(plotDict6)["Value"]["plot"]

        self.finish(
            {
                "success": "true",
                "result": [
                    {"Type": "Accounting", "src": image1},
                    {"Type": "Accounting", "src": image2},
                    {"Type": "Accounting", "src": image3},
                    {"Type": "Monitoring", "src": image4},
                    {"Type": "Accounting", "src": image5},
                    {"Type": "Accounting", "src": image6},
                ],
                "total": 6,
            }
        )

    def getPlotDict(self, siteName, grouping, reportName, typeName, plotTitle=None, status=None):

        plotDict = {
            "condDict": {
                # DIRAC.AccountingSystem.Client.Types.DataOperation class use 'ExecutionSite' key instead 'Site'
                "ExecutionSite" if typeName == "DataOperation" else "Site": [siteName],
                "grouping": [grouping],
            },
            "extraArgs": {"lastSeconds": 86400},
            "grouping": grouping,
            "reportName": reportName,
            "typeName": typeName,
            "_plotTitle": plotTitle,
        }

        if plotTitle is not None:
            plotDict["extraArgs"]["plotTitle"] = plotTitle
        if status is not None:
            plotDict["condDict"]["Status"] = [status]

        return plotDict

    def __requestParams(self):
        """
        We receive the request and we parse it, in this case, we are doing nothing,
        but it can be certainly more complex.
        """

        gLogger.always("!!!  PARAMS: ", repr(self.request.arguments))

        responseParams = {
            "name": None,
            "elementType": None,
            "statusType": None,
            "status": None,
            "action": None,
            "tokenOwner": None,
        }

        for key in responseParams:
            value = self.get_argument(key, None)
            if value:
                responseParams[key] = list(json.loads(value))

        return responseParams



import os
import json
import tornado
import tempfile

from DIRAC import gConfig
from DIRAC.Core.DISET.RPCClient import RPCClient
from DIRAC.Core.DISET.TransferClient import TransferClient
from DIRAC.Core.Utilities import Time, DEncode

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class ActivityMonitorHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getActivityData(self):
        try:
            start = int(self.get_argument("start"))
        except BaseException:
            start = 0
        try:
            limit = int(self.get_argument("limit"))
        except BaseException:
            limit = 0

        try:
            sortField = self.get_argument("sortField").replace("_", ".")
            sortDir = self.get_argument("sortDirection")
            sort = [(sortField, sortDir)]
        except BaseException:
            sort = []

        rpcClient = RPCClient("Framework/Monitoring")
        retVal = yield self.threadTask(rpcClient.getActivitiesContents, {}, sort, start, limit)

        if not retVal["OK"]:
            self.finish({"success": "false", "result": [], "total": -1, "error": retVal["Message"]})
            return

        svcData = retVal["Value"]
        callback = {"success": "true", "total": svcData["TotalRecords"], "result": []}
        now = Time.toEpoch()
        for record in svcData["Records"]:
            formatted = {}
            for i in range(len(svcData["Fields"])):
                formatted[svcData["Fields"][i].replace(".", "_")] = record[i]
            if "activities_lastUpdate" in formatted:
                formatted["activities_lastUpdate"] = now - int(formatted["activities_lastUpdate"])
            callback["result"].append(formatted)

        self.finish(callback)

    def __dateToSecs(self, timeVar):
        dt = Time.fromString(timeVar)
        return int(Time.toEpoch(dt))

    @asyncGen
    def web_plotView(self):

        plotRequest = {}
        try:
            if "id" not in self.request.arguments:
                self.finish({"success": "false", "error": "Missing viewID in plot request"})
                return
            plotRequest["id"] = self.get_argument("id")
            if "size" not in self.request.arguments:
                self.finish({"success": "false", "error": "Missing plotsize in plot request"})
                return
            plotRequest["size"] = int(self.get_argument("size"))

            timespan = int(self.get_argument("timespan"))
            if timespan < 0:
                toSecs = self.__dateToSecs(self.get_argument("toDate"))
                fromSecs = self.__dateToSecs(self.get_argument("fromDate"))
            else:
                toSecs = int(Time.toEpoch())
                fromSecs = toSecs - timespan
            plotRequest["fromSecs"] = fromSecs
            plotRequest["toSecs"] = toSecs
            if "varData" in self.request.arguments:
                plotRequest["varData"] = dict(json.loads(self.get_argument("varData")))
        except Exception as e:
            self.finish({"success": "false", "error": "Error while processing plot parameters: %s" % str(e)})
            return

        rpcClient = RPCClient("Framework/Monitoring")
        retVal = yield self.threadTask(rpcClient.plotView, plotRequest)

        if retVal["OK"]:
            self.finish({"success": "true", "data": retVal["Value"]})
        else:
            self.finish({"success": "false", "error": retVal["Message"]})

    @asyncGen
    def web_getStaticPlotViews(self):
        rpcClient = RPCClient("Framework/Monitoring")
        retVal = yield self.threadTask(rpcClient.getViews, True)
        if not retVal["OK"]:
            self.finish({"success": "false", "error": retVal["Message"]})
        else:
            self.finish({"success": "true", "result": retVal["Value"]})

    @asyncGen
    def web_getPlotImg(self):
        """
        Get plot image
        """
        callback = {}
        if "file" not in self.request.arguments:
            callback = {"success": "false", "error": "Maybe you forgot the file?"}
            self.finish(callback)
            return
        plotImageFile = self.get_argument("file")
        # Prevent directory traversal
        plotImageFile = os.path.normpath("/" + plotImageFile).lstrip("/")

        transferClient = TransferClient("Framework/Monitoring")
        tempFile = tempfile.TemporaryFile()
        retVal = yield self.threadTask(transferClient.receiveFile, tempFile, plotImageFile)
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Message"]}
            self.finish(callback)
            return
        tempFile.seek(0)
        data = tempFile.read()
        self.finishWithImage(data, plotImageFile)

    @asyncGen
    def web_queryFieldValue(self):
        """
        Query a value for a field
        """
        fieldQuery = self.get_argument("queryField")
        definedFields = json.loads(self.get_argument("selectedFields"))
        rpcClient = RPCClient("Framework/Monitoring")
        result = yield self.threadTask(rpcClient.queryField, fieldQuery, definedFields)
        if "rpcStub" in result:
            del result["rpcStub"]

        if result["OK"]:
            self.finish({"success": "true", "result": result["Value"]})
        else:
            self.finish({"success": "false", "error": result["Message"]})

    @asyncGen
    def web_deleteActivities(self):
        try:
            webIds = self.get_argument("ids").split(",")
        except Exception as e:
            self.finish({"success": "false", "error": "No valid id's specified"})
            return

        idList = []
        for webId in webIds:
            try:
                idList.append([int(field) for field in webId.split(".")])
            except Exception as e:
                self.finish({"success": "false", "error": "Error while processing arguments: %s" % str(e)})
                return

        rpcClient = RPCClient("Framework/Monitoring")

        retVal = yield self.threadTask(rpcClient.deleteActivities, idList)

        if "rpcStub" in retVal:
            del retVal["rpcStub"]

        if retVal["OK"]:
            self.finish({"success": "true"})
        else:
            self.finish({"success": "false", "error": retVal["Message"]})

    @asyncGen
    def web_tryView(self):
        """
        Try plotting graphs for a view
        """
        try:
            plotRequest = json.loads(self.get_argument("plotRequest"))
            if "timeLength" in self.request.arguments:
                timeLength = self.get_argument("timeLength")
                toSecs = int(Time.toEpoch())
                if timeLength == "hour":
                    fromSecs = toSecs - 3600
                elif timeLength == "day":
                    fromSecs = toSecs - 86400
                elif timeLength == "month":
                    fromSecs = toSecs - 2592000
                elif fromSecs == "year":
                    fromDate = toSecs - 31104000
                else:
                    self.finish({"success": "false", "error": "Time length value not valid"})
                    return
            else:
                fromDate = self.get_argument("fromDate")
                toDate = self.get_argument("toDate")
                fromSecs = self.__dateToSecs(fromDate)
                toSecs = self.__dateToSecs(toDate)
        except Exception as e:
            self.finish({"success": "false", "error": "Error while processing plot parameters: %s" % str(e)})
            return

        rpcClient = RPCClient("Framework/Monitoring")
        requestStub = DEncode.encode(plotRequest)
        retVal = yield self.threadTask(rpcClient.tryView, fromSecs, toSecs, requestStub)
        if not retVal["OK"]:
            self.finish({"success": "false", "error": retVal["Message"]})
            return

        self.finish({"success": "true", "images": retVal["Value"], "stub": requestStub})

    @asyncGen
    def web_saveView(self):
        """
        Save a view
        """
        try:
            plotRequest = json.loads(self.get_argument("plotRequest"))
            viewName = self.get_argument("viewName")
        except Exception as e:
            self.finish({"success": "false", "error": "Error while processing plot parameters: %s" % str(e)})
            return
        rpcClient = RPCClient("Framework/Monitoring")
        requestStub = DEncode.encode(plotRequest)
        result = yield self.threadTask(rpcClient.saveView, viewName, requestStub)
        if "rpcStub" in result:
            del result["rpcStub"]

        self.finish({"success": "true"})

    def __getSections(self, path):

        result = []

        retVal = gConfig.getSections("/DIRAC/Setups")
        if retVal["OK"]:
            setups = [i.split("-")[-1] for i in retVal["Value"]]
        setup = self.getUserSetup().split("-")[-1]
        leaf = True if path.find("Agents") != -1 or path.find("Services") != -1 else False
        retVal = gConfig.getSections(path)

        if retVal["OK"]:
            records = retVal["Value"]
            for i in records:
                if i in setups and i != setup:
                    continue
                if i == setup:
                    path = "%s/%s" % (path, i)
                    result = self.__getSections(path)

                if i not in [setup, "Databases", "URLs"]:

                    id = "%s/%s" % (path, i)
                    components = path.split("/")
                    if len(components) > 2:
                        componentName = "%s/%s" % (components[2], i)
                    else:
                        componentName = i
                    result += [{"text": i, "qtip": "Systems", "leaf": leaf, "component": componentName, "id": id}]

        return result

    @asyncGen
    def web_getDynamicPlotViews(self):
        """
        It retrieves the systems from the CS.
        """
        nodes = []
        path = self.get_argument("node")

        result = self.__getSections(path)
        for i in result:
            nodes += [i]

        result = tornado.escape.json_encode(nodes)
        self.finish(result)



import os
import ssl
import sys
import signal
import tornado.web
import tornado.process
import tornado.httpserver
import tornado.autoreload

from DIRAC import gLogger, S_OK
from WebAppDIRAC.Core.HandlerMgr import HandlerMgr
from WebAppDIRAC.Core.TemplateLoader import TemplateLoader
from WebAppDIRAC.Lib.SessionData import SessionData
from WebAppDIRAC.Lib import Conf

# If we are running with python3, Tornado will use asyncio,
# and we have to convince it to let us run in a different thread
# Doing this ensures a consistent behavior between py2 and py3
# see https://www.tornadoweb.org/en/stable/asyncio.html#tornado.platform.asyncio.AnyThreadEventLoopPolicy
import asyncio
from tornado.platform.asyncio import AnyThreadEventLoopPolicy

asyncio.set_event_loop_policy(AnyThreadEventLoopPolicy())


class App:
    def __init__(self, handlersLoc="WebApp.handler"):
        self.__handlerMgr = HandlerMgr(handlersLoc, Conf.rootURL())
        self.__servers = {}
        self.log = gLogger.getSubLogger("Web")

    def _logRequest(self, handler):
        status = handler.get_status()
        if status < 400:
            logm = self.log.notice
        elif status < 500:
            logm = self.log.warn
        else:
            logm = self.log.error
        request_time = 1000.0 * handler.request.request_time()
        logm("%d %s %.2fms" % (status, handler._request_summary(), request_time))

    def __reloadAppCB(self):
        gLogger.notice("\n !!!!!! Reloading web app...\n")

    def stopChildProcesses(self, sig, frame):
        """
        It is used to properly stop tornado when more than one process is used.
        In principle this is doing the job of runsv....
        :param int sig: the signal sent to the process
        :param object frame: execution frame which contains the child processes
        """
        # tornado.ioloop.IOLoop.instance().add_timeout(time.time()+5, sys.exit)
        for child in frame.f_locals.get("children", []):
            gLogger.info("Stopping child processes: %d" % child)
            os.kill(child, signal.SIGTERM)
        # tornado.ioloop.IOLoop.instance().stop()
        # gLogger.info('exit success')
        sys.exit(0)

    def getAppToDict(self, port=None):
        """Load Web portals

        :return: S_OK(dict)/S_ERROR()
        """
        app = {"port": port or Conf.HTTPSPort()}
        # Calculating routes
        result = self.__handlerMgr.getRoutes()
        if not result["OK"]:
            return result
        app["routes"] = result["Value"]
        # Initialize the session data
        SessionData.setHandlers(self.__handlerMgr.getHandlers()["Value"])
        # Create the app
        tLoader = TemplateLoader(self.__handlerMgr.getPaths("template"))
        app["settings"] = dict(
            debug=Conf.devMode(),
            template_loader=tLoader,
            cookie_secret=str(Conf.cookieSecret()),
            log_function=self._logRequest,
        )
        return S_OK(app)

    def bootstrap(self):
        """
        Configure and create web app
        """
        self.log.always("\n ====== Starting DIRAC web app ====== \n")

        # Calculating routes
        result = self.__handlerMgr.getRoutes()
        if not result["OK"]:
            return result
        routes = result["Value"]
        # Initialize the session data
        SessionData.setHandlers(self.__handlerMgr.getHandlers()["Value"])
        # Create the app
        tLoader = TemplateLoader(self.__handlerMgr.getPaths("template"))
        kw = dict(
            debug=Conf.devMode(),
            template_loader=tLoader,
            cookie_secret=str(Conf.cookieSecret()),
            log_function=self._logRequest,
            autoreload=Conf.numProcesses() < 2,
        )

        # please do no move this lines. The lines must be before the fork_processes
        signal.signal(signal.SIGTERM, self.stopChildProcesses)
        signal.signal(signal.SIGINT, self.stopChildProcesses)

        # Check processes if we're under a load balancert
        if Conf.balancer() and Conf.numProcesses() not in (0, 1):
            tornado.process.fork_processes(Conf.numProcesses(), max_restarts=0)
            kw["debug"] = False
        # Debug mode?
        if kw["debug"]:
            self.log.info("Configuring in developer mode...")
        # Configure tornado app
        self.__app = tornado.web.Application(routes, **kw)
        port = Conf.HTTPPort()
        self.log.notice("Configuring HTTP on port %s" % port)
        # Create the web servers
        srv = tornado.httpserver.HTTPServer(self.__app, xheaders=True)
        srv.listen(port)
        self.__servers[("http", port)] = srv

        Conf.generateRevokedCertsFile()  # it is used by nginx....

        if Conf.HTTPS():
            self.log.notice("Configuring HTTPS on port %s" % Conf.HTTPSPort())
            sslops = dict(
                certfile=Conf.HTTPSCert(),
                keyfile=Conf.HTTPSKey(),
                cert_reqs=ssl.CERT_OPTIONAL,
                ca_certs=Conf.generateCAFile(),
                ssl_version=ssl.PROTOCOL_TLSv1_2,
            )

            sslprotocol = str(Conf.SSLProtocol())
            aviableProtocols = [i for i in dir(ssl) if i.find("PROTOCOL") == 0]
            if sslprotocol and sslprotocol != "":
                if sslprotocol in aviableProtocols:
                    sslops["ssl_version"] = getattr(ssl, sslprotocol)
                else:
                    message = "%s protocol is not provided." % sslprotocol
                    message += "The following protocols are provided: %s" % str(aviableProtocols)
                    gLogger.warn(message)

            self.log.debug(" - %s" % "\n - ".join(["%s = %s" % (k, sslops[k]) for k in sslops]))
            srv = tornado.httpserver.HTTPServer(self.__app, ssl_options=sslops, xheaders=True)
            port = Conf.HTTPSPort()
            srv.listen(port)
            self.__servers[("https", port)] = srv
        else:
            # when NGINX is used then the Conf.HTTPS return False, it means tornado
            # does not have to be configured using 443 port
            Conf.generateCAFile()  # if we use Nginx we have to generate the cas as well...
        return result

    def run(self):
        """
        Start web servers
        """
        bu = Conf.rootURL().strip("/")
        urls = []
        for proto, port in self.__servers:
            urls.append("%s://0.0.0.0:%s/%s/" % (proto, port, bu))
        self.log.always("Listening on %s" % " and ".join(urls))
        tornado.autoreload.add_reload_hook(self.__reloadAppCB)
        tornado.ioloop.IOLoop.instance().start()



import json
import pprint
from urllib.parse import urlparse

from tornado.escape import xhtml_escape
from tornado import template

from DIRAC import gLogger
from DIRAC.Core.Tornado.Server.private.BaseRequestHandler import TornadoResponse

from WebAppDIRAC.Lib import Conf
from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler, WErr


class RootHandler(WebHandler):

    DEFAULT_AUTHORIZATION = "all"
    DEFAULT_LOCATION = "/"
    SUPPORTED_METHODS = ("GET",)

    def web_changeGroup(self, to):
        """Change group

        :param str to: group name

        :return: TornadoResponse()
        """
        return TornadoResponse().redirect(self.__change(group=to))  # pylint: disable=no-member

    def web_changeSetup(self, to):
        """Change setup

        :param str to: setup name

        :return: TornadoResponse()
        """
        return TornadoResponse().redirect(self.__change(setup=to))  # pylint: disable=no-member

    def __change(self, setup=None, group=None):
        """Generate URL to change setup/group

        :param str setup: setup name
        :param str group: group name

        :return: str
        """
        url = [
            Conf.rootURL().strip("/"),
            "s:%s" % (setup or self.getUserSetup()),
            "g:%s" % (group or self.getUserGroup() or "anon"),
        ]
        qs = False
        if "Referer" in self.request.headers:
            o = urlparse(self.request.headers["Referer"])
            url.append("?%s" % o.query)
        return "/%s" % "/".join(url)

    def web_getConfigData(self, **kwargs):
        """Get session data

        :return: dict
        """
        return self.getSessionData()

    def web_logout(self, **kwargs):
        """Logout"""
        token = self.get_secure_cookie("session_id")
        if token:
            token = json.loads(token)
            if token.get("refresh_token"):
                result = self._idps.getIdProvider("DIRACWeb")
                if result["OK"]:
                    cli = result["Value"]
                    cli.token = token
                    cli.revokeToken(token["refresh_token"])

    def finish_logout(self):
        """Finish logout process"""
        self.clear_cookie("session_id")
        self.set_cookie("authGrant", "Visitor")
        self.redirect("/DIRAC")

    def web_login(self, provider, next="/DIRAC", **kwargs):
        """Start authorization flow

        :param str provider: provider name
        :param str next: current URI

        :return: TornadoResponse()
        """
        result = self._idps.getIdProvider("DIRACWeb")
        if not result["OK"]:
            raise WErr(500, result["Message"])
        cli = result["Value"]
        cli.scope = ""
        if provider:
            cli.metadata["authorization_endpoint"] = "%s/%s" % (cli.get_metadata("authorization_endpoint"), provider)

        uri, state, session = cli.submitNewSession()

        # Save authorisation session
        session.update(dict(state=state, provider=provider, next=next))

        resp = TornadoResponse()
        # pylint: disable=no-member
        resp.set_secure_cookie("webauth_session", json.dumps(session), secure=True, httponly=True)

        # Redirect to authorization server
        resp.set_cookie("authGrant", "Visitor")  # pylint: disable=no-member
        return resp.redirect(uri)  # pylint: disable=no-member

    def web_loginComplete(self, code, state, **kwargs):
        """Finishing authoriation flow

        :param str code: code
        :param str state: state

        :return: TornadoResponse()
        """
        t = template.Template(
            """<!DOCTYPE html>
    <html>
      <head>
        <title>Authentication</title>
        <meta charset="utf-8" />
      </head>
      <body>
        {{message}}
        <script>
          sessionStorage.setItem("access_token", "{{access_token}}");
          window.location = "{{next}}";
        </script>
      </body>
    </html>"""
        )
        resp = TornadoResponse()
        authSession = self.get_secure_cookie("webauth_session")
        if not authSession:
            return resp.redirect("/")  # pylint: disable=no-member
        authSession = json.loads(authSession)

        result = self._idps.getIdProvider("DIRACWeb")
        if not result["OK"]:
            # pylint: disable=no-member
            resp.finish(t.generate(next=authSession["next"], access_token="", message=result["Message"]).decode())
            return resp
        cli = result["Value"]

        result = cli.fetchToken(authorization_response=self.request.uri, code_verifier=authSession.get("code_verifier"))
        if not result["OK"]:
            # pylint: disable=no-member
            resp.finish(t.generate(next=authSession["next"], access_token="", message=result["Message"]).decode())
            return resp
        token = result["Value"]

        # Remove authorisation session.
        resp.clear_cookie("webauth_session")  # pylint: disable=no-member

        # Create session to work through portal
        self.log.debug("Tokens received:\n", pprint.pformat(token))
        # pylint: disable=no-member
        resp.set_secure_cookie("session_id", json.dumps(dict(token)), secure=True, httponly=True)
        resp.set_cookie("authGrant", "Session")  # pylint: disable=no-member

        result = cli.researchGroup()
        if not result["OK"]:
            # pylint: disable=no-member
            return resp.finish(
                t.generate(next=authSession["next"], access_token="", message=result["Message"]).decode()
            )
        group = result["Value"].get("group")

        url = "/".join([Conf.rootURL().strip("/"), "s:%s" % self.getUserSetup(), "g:%s" % group])
        nextURL = "/%s/?%s" % (url, urlparse(authSession["next"]).query)
        # Save token and go to main page
        # with document('DIRAC authentication') as html:
        #   dom.div('Authorization is done.',
        #           style='display:flex;justify-content:center;align-items:center;padding:28px;font-size:28px;')
        #   dom.script("sessionStorage.setItem('access_token','%s');window.location='%s'" % (access_token, nextURL),
        #              type="text/javascript")
        # return template.Template(html.render()).generate()
        # pylint: disable=no-member
        resp.finish(
            t.generate(next=nextURL, access_token=token["access_token"], message="Authorization is done").decode()
        )
        return resp

    def web_index(self, *, url_state="", theme="crisp", open_app="", **kwargs):
        """Index method

        :param str url_state: url state
        :param str theme: selected theme name, default "crisp"

        :return: TornadoResponse()
        """
        # Render base template
        data = self.getSessionData()

        welcome = ""
        welcomeFile = Conf.getWelcome()
        if welcomeFile:
            try:
                with open(welcomeFile, "r") as f:
                    welcome = f.read().replace("\n", "")
            except Exception:
                gLogger.warn("Welcome page not found here: %s" % welcomeFile)

        # pylint: disable=no-member
        return TornadoResponse().render(
            "root.tpl",
            _dev=Conf.devMode(),
            logo=data["baseURL"] + Conf.getLogo(),
            view="tabs",
            theme=theme.lower(),
            title=Conf.getTitle(),
            welcome=welcome,
            iconUrl=data["baseURL"] + Conf.getIcon(),
            open_app=open_app.strip(),
            base_url=data["baseURL"],
            root_url=Conf.rootURL(),
            url_state=xhtml_escape(url_state),
            http_port=Conf.HTTPPort(),
            https_port=Conf.HTTPSPort(),
            extensions=data["extensions"],
            credentials=data["user"],
            ext_version=data["extVersion"],
            debug_level=str(gLogger.getLevel()).lower(),
            bugReportURL=Conf.bugReportURL(),
            backgroundImage=data["baseURL"] + Conf.getBackgroud(),
            auth_client_settings=data["configuration"].get("AuthorizationClient", {}),
        )



import os

from DIRAC import gConfig, gLogger
from DIRAC.Core.Utilities import List
from DIRAC.Core.DISET.AuthManager import AuthManager
from DIRAC.ConfigurationSystem.Client.Helpers import Registry
from DIRAC.Core.Utilities.Extensions import extensionsByPriority

# from DIRAC.FrameworkSystem.Client.ProxyManagerClient import gProxyManager

from WebAppDIRAC.Lib import Conf

DEFAULT_SCHEMA = [
    [
        "Tools",
        [
            ["app", "Application Wizard", "DIRAC.ApplicationWizard"],
            ["app", "Job Launchpad", "DIRAC.JobLaunchpad"],
            ["app", "Notepad", "DIRAC.Notepad"],
            ["app", "Proxy Upload", "DIRAC.ProxyUpload"],
        ],
    ],
    [
        "Applications",
        [
            ["app", "Accounting", "DIRAC.Accounting"],
            ["app", "Activity Monitor", "DIRAC.ActivityMonitor"],
            ["app", "Configuration Manager", "DIRAC.ConfigurationManager"],
            ["app", "Job Monitor", "DIRAC.JobMonitor"],
            ["app", "Downtimes", "DIRAC.Downtimes"],
            ["app", "File Catalog", "DIRAC.FileCatalog"],
            ["app", "Job Monitor", "DIRAC.JobMonitor"],
            ["app", "Job Summary", "DIRAC.JobSummary"],
            ["app", "Pilot Monitor", "DIRAC.PilotMonitor"],
            ["app", "Pilot Summary", "DIRAC.PilotSummary"],
            ["app", "Proxy Manager", "DIRAC.ProxyManager"],
            ["app", "Public State Manager", "DIRAC.PublicStateManager"],
            ["app", "Registry Manager", "DIRAC.RegistryManager"],
            ["app", "Request Monitor", "DIRAC.RequestMonitor"],
            ["app", "Resource Summary", "DIRAC.ResourceSummary"],
            ["app", "Site Summary", "DIRAC.SiteSummary"],
            ["app", "Space Occupancy", "DIRAC.SpaceOccupancy"],
            ["app", "System Administration", "DIRAC.SystemAdministration"],
            ["app", "Transformation Monitor", "DIRAC.TransformationMonitor"],
        ],
    ],
]


class SessionData:

    __handlers = {}
    __groupMenu = {}
    __extensions = []
    __extVersion = "ext-6.2.0"
    __configuration = {}

    @classmethod
    def setHandlers(cls, handlers):
        """Set handlers

        :param dict handlers: handlers
        """
        cls.__handlers = {}
        for k in handlers:
            handler = handlers[k]
            cls.__handlers[handler.LOCATION.strip("/")] = handler
        # Calculate extensions
        cls.__extensions = extensionsByPriority()
        for ext in ["DIRAC", "WebAppDIRAC"]:
            if ext in cls.__extensions:
                cls.__extensions.append(cls.__extensions.pop(cls.__extensions.index(ext)))

    def __init__(self, credDict, setup):
        self.__credDict = credDict
        self.__setup = setup

    def __isGroupAuthApp(self, appLoc):
        """The method checks if the application is authorized for a certain user group

        :param str appLoc It is the application name for example: DIRAC.JobMonitor

        :return bool -- if the handler is authorized to the user returns True otherwise False
        """
        handlerLoc = "/".join(List.fromChar(appLoc, ".")[1:])
        if not handlerLoc:
            gLogger.error("Application handler does not exists:", appLoc)
            return False
        if handlerLoc not in self.__handlers:
            gLogger.error("Handler %s required by %s does not exist!" % (handlerLoc, appLoc))
            return False
        handler = self.__handlers[handlerLoc]
        auth = AuthManager(Conf.getAuthSectionForHandler(handlerLoc))
        gLogger.info("Authorization: %s -> %s" % (dict(self.__credDict), handler.AUTH_PROPS))
        return auth.authQuery("", dict(self.__credDict), handler.AUTH_PROPS)

    def __generateSchema(self, base, path):
        """Generate a menu schema based on the user credentials

        :param str base: base
        :param str path: path

        :return: list
        """
        # Calculate schema
        schema = []
        fullName = "%s/%s" % (base, path)
        result = gConfig.getSections(fullName)
        if not result["OK"]:
            return schema
        sectionsList = result["Value"]
        for sName in sectionsList:
            subSchema = self.__generateSchema(base, "%s/%s" % (path, sName))
            if subSchema:
                schema.append((sName, subSchema))
        result = gConfig.getOptions(fullName)
        if not result["OK"]:
            return schema
        optionsList = result["Value"]
        for opName in optionsList:
            opVal = gConfig.getValue("%s/%s" % (fullName, opName))
            if opVal.startswith("link|"):
                schema.append(("link", opName, opVal[5:]))  # pylint: disable=unsubscriptable-object
                continue
            if self.__isGroupAuthApp(opVal):
                schema.append(("app", opName, opVal))
        return schema

    def __generateDefaultSchema(self):
        """Generate a menu schema based on the user credentials

        :param str base: base
        :param str path: path

        :return: list
        """
        schema = []
        for section, apps in DEFAULT_SCHEMA:
            appList = []
            for app in apps:
                if self.__isGroupAuthApp(app[-1]):
                    appList.append(app)
            if appList:
                schema.append((section, appList))
        return schema

    def __getGroupMenu(self):
        """Load the schema from the CS and filter based on the group

        :param dict cfg: dictionary with current configuration

        :return: list
        """
        menuSection = "%s/Schema" % (Conf.BASECS)
        # Somebody coming from HTTPS and not with a valid group
        group = self.__credDict.get("group", "")
        # Cache time!
        if group not in self.__groupMenu:
            result = gConfig.getSections(menuSection)
            if not result["OK"] or not result["Value"]:
                self.__groupMenu[group] = self.__generateDefaultSchema()
            else:
                self.__groupMenu[group] = self.__generateSchema(menuSection, "")
        return self.__groupMenu[group]

    @classmethod
    def getWebAppPath(cls):
        """Get WebApp path

        :return: str
        """
        return os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "WebApp")

    @classmethod
    def getExtJSVersion(cls):
        """Get ExtJS version

        :return: str
        """
        return cls.__extVersion

    @classmethod
    def getWebConfiguration(cls):
        """Get WebApp configuration

        :return: dict
        """
        result = gConfig.getOptionsDictRecursively("/WebApp")
        if not cls.__configuration and result["OK"]:
            cls.__configuration = result["Value"]
        return cls.__configuration

    def getData(self):
        """Return session data

        :return: dict
        """
        data = {
            "configuration": self.getWebConfiguration(),
            "menu": self.__getGroupMenu(),
            "user": self.__credDict,
            "validGroups": [],
            # 'groupsStatuses': '',
            "setup": self.__setup,
            "validSetups": gConfig.getSections("/DIRAC/Setups")["Value"],
            "extensions": self.__extensions,
            "extVersion": self.getExtJSVersion(),
        }
        # Add valid groups if known
        username = self.__credDict.get("username", "anonymous")
        if username != "anonymous":
            result = Registry.getGroupsForUser(username)
            if not result["OK"]:
                return result
            data["validGroups"] = result["Value"]
            # result = gProxyManager.getGroupsStatusByUsername(username)  # pylint: disable=no-member
            # if result['OK']:
            #   data['groupsStatuses'] = result['Value']
        # Calculate baseURL
        baseURL = [Conf.rootURL().strip("/"), "s:%s" % data["setup"], "g:%s" % self.__credDict.get("group", "")]
        data["baseURL"] = "/%s" % "/".join(baseURL)
        return data



#!/usr/bin/env python
import os
import sys

from DIRAC import gConfig, S_OK
from DIRAC.Core.Utilities.Extensions import extensionsByPriority, getExtensionMetadata
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC.ConfigurationSystem.Client.LocalConfiguration import LocalConfiguration
from DIRAC.FrameworkSystem.Client.Logger import gLogger

from WebAppDIRAC.Core.App import App

__RCSID__ = "$Id$"


def _createStaticSymlinks(targetDir):
    """Create symlinks to static web content

    This method is used to populate the directory specified in the local
    configuration under ``/WebApp/StaticResourceLinkDir`` so that nginx can
    serve this content from a consistent location.

    :params str targetDir: The directory in which to create the symlinks
    """
    if not os.path.isdir(targetDir):
        os.makedirs(targetDir)
    extensions = extensionsByPriority()
    for extension in extensions:
        metadata = getExtensionMetadata(extension)
        staticDirs = metadata.get("web_resources", {}).get("static", [])

        for i, path in enumerate(sorted(staticDirs)):
            destination = os.path.join(targetDir, extension)
            # If there is more than one suffix append a counter
            if i >= 1:
                destination += "-%s" % i
            gLogger.notice("    creating symlink from", "%s to %s" % (path, destination))
            if os.path.islink(destination):
                if path == os.readlink(destination):
                    # The link is already up to date
                    continue
                os.unlink(destination)
            os.symlink(path, destination)

    # TODO: Check /etc/nginx/conf.d/site.conf and warn if it's inconsistent?


def _checkDIRACVersion():
    """Validate a compatible DIRAC version is installed

    In order to build WebApp extensions it is necessary to install the vanilla
    WebApp during the compilation of the javascrip sources. The dependency on DIRAC
    makes this unreliable as M2Crypto and gfal2 are both hard to install.

    Instead, the DIRAC dependency is only included when installing the "server"
    extra however this makes it possible to install incompatible version of DIRAC.
    To avoid hard-to-debug failures we inspect the metadata when launching the
    service and refuse to start if the DIRAC version is incompatible.
    """
    from importlib.metadata import requires, version  # pylint: disable=import-error,no-name-in-module
    from packaging.requirements import Requirement  # pylint: disable=no-name-in-module

    deps = [Requirement(x) for x in requires("WebAppDIRAC")]
    deps = [x for x in deps if x.name.lower() == "dirac"]
    if len(deps) != 1:
        raise NotImplementedError("This shouldn't be possible: %r" % deps)
    dirac_version = version("DIRAC")
    dirac_spec = deps[0].specifier
    if dirac_version not in dirac_spec:
        raise RuntimeError(
            "WebAppDIRAC %s requires %s but %s is incompatible" % (version("WebAppDIRAC"), dirac_version, dirac_spec)
        )


@DIRACScript()
def main():
    def ignoreIncompatible(op):
        gConfig.setOptionValue("/WebApp/IgnoreVersionCheck", "True")
        return S_OK()

    def disableDevMode(op):
        gConfig.setOptionValue("/WebApp/DevelopMode", "False")
        return S_OK()

    def setHandlers(op):
        localCfg.handlersLoc = op
        gLogger.notice("SET handlersLoc to ", localCfg.handlersLoc)
        return S_OK()

    localCfg = LocalConfiguration()

    localCfg.handlersLoc = "WebApp.handler"
    localCfg.setConfigurationForWeb("WebApp")
    localCfg.addMandatoryEntry("/DIRAC/Setup")
    localCfg.addDefaultEntry("/DIRAC/Security/UseServerCertificate", "yes")
    localCfg.addDefaultEntry("LogLevel", "INFO")
    localCfg.addDefaultEntry("LogColor", True)
    localCfg.registerCmdOpt("i", "--ignore-incompatible", "Ignore incompatible version.", ignoreIncompatible)
    localCfg.registerCmdOpt("p", "production", "Enable production mode", disableDevMode)
    localCfg.registerCmdOpt(
        "S:",
        "set_handlers_location=",
        "Specify path(s) to handlers, for ex. 'OAuthDIRAC.FrameworkSystem.Utilities'",
        setHandlers,
    )

    result = localCfg.loadUserData()
    if not result["OK"]:
        gLogger.initialize("WebApp", "/")
        gLogger.fatal("There were errors when loading configuration", result["Message"])
        sys.exit(1)

    if gConfig.getOption("/WebApp/IgnoreVersionCheck").get("Value", "").lower() not in ["yes", "true"]:
        _checkDIRACVersion()

    result = gConfig.getOption("/WebApp/StaticResourceLinkDir")
    if result["OK"]:
        gLogger.notice("Creating symlinks to static resources")
        _createStaticSymlinks(result["Value"])
    else:
        gLogger.warn("Not creating symlinks to static resources", result["Message"])

    gLogger.notice("Set next path(s): ", localCfg.handlersLoc)
    app = App(handlersLoc=localCfg.handlersLoc)
    result = app.bootstrap()
    if not result["OK"]:
        gLogger.fatal(result["Message"])
        sys.exit(1)
    app.run()


if __name__ == "__main__":
    main()



import json

from DIRAC import gConfig, S_OK, gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.Core.Utilities.Graphs.Palette import Palette
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getUsernameForDN
from DIRAC.WorkloadManagementSystem.Client.PilotManagerClient import PilotManagerClient
from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class PilotMonitorHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getPilotData(self):
        req = self.__request()

        result = yield self.threadTask(
            PilotManagerClient().getPilotMonitorWeb, req, self.globalSort, self.pageNumber, self.numberOfJobs
        )

        if not result["OK"]:
            self.finish({"success": "false", "result": [], "total": 0, "error": result["Message"]})
            return

        result = result["Value"]

        if "TotalRecords" not in result:
            self.finish({"success": "false", "result": [], "total": -1, "error": "Data structure is corrupted"})
            return

        if not (result["TotalRecords"] > 0):
            self.finish(
                {"success": "false", "result": [], "total": 0, "error": "There were no data matching your selection"}
            )
            return

        if not ("ParameterNames" in result and "Records" in result):
            self.finish({"success": "false", "result": [], "total": -1, "error": "Data structure is corrupted"})
            return

        if not (len(result["ParameterNames"]) > 0):
            self.finish({"success": "false", "result": [], "total": -1, "error": "ParameterNames field is missing"})
            return

        if not (len(result["Records"]) > 0):
            self.finish({"success": "false", "result": [], "total": 0, "Message": "There are no data to display"})
            return

        callback = []
        jobs = result["Records"]
        head = result["ParameterNames"]
        headLength = len(head)

        for i in jobs:
            tmp = {}
            for j in range(0, headLength):
                if j == 12:
                    if i[j] == 0:
                        i[j] = "-"
                tmp[head[j]] = i[j]
            callback.append(tmp)
        total = result["TotalRecords"]
        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
        if "Extras" in result:
            extra = result["Extras"]
            callback = {"success": "true", "result": callback, "total": total, "extra": extra, "date": timestamp}
        else:
            callback = {"success": "true", "result": callback, "total": total, "date": timestamp}

        self.finish(callback)

    @asyncGen
    def web_getSelectionData(self):
        callback = {}

        if len(self.request.arguments) > 0:
            tmp = {self.get_argument(i).replace('"', "") for i in self.request.arguments}
            callback["extra"] = list(tmp)

        result = yield self.threadTask(PilotManagerClient().getPilotMonitorSelectors)

        if result["OK"]:
            result = result["Value"]

            if "Status" in result and len(result["Status"]) > 0:
                status = []
                for i in result["Status"]:
                    status.append([str(i)])
            else:
                status = [["Nothing to display"]]
            callback["status"] = status

            if "GridType" in result and len(result["GridType"]) > 0:
                gridtype = []
                for i in result["GridType"]:
                    gridtype.append([str(i)])
            else:
                gridtype = [["Nothing to display"]]
            callback["gridtype"] = gridtype

            if "OwnerGroup" in result and len(result["OwnerGroup"]) > 0:
                ownerGroup = []
                for i in result["OwnerGroup"]:
                    ownerGroup.append([str(i)])
            else:
                ownerGroup = [["Nothing to display"]]
            callback["ownerGroup"] = ownerGroup

            if "DestinationSite" in result and len(result["DestinationSite"]) > 0:
                ce = []
                for i in result["DestinationSite"]:
                    ce.append([str(i)])
            else:
                ce = [["Nothing to display"]]
            callback["computingElement"] = ce

            if "GridSite" in result and len(result["GridSite"]) > 0:
                tier1 = gConfig.getValue("/WebApp/PreferredSites", [])
                site = []
                s = list(result["GridSite"])
                for i in tier1:
                    site.append([str(i)])
                for i in s:
                    if i not in tier1:
                        site.append([str(i)])
            else:
                site = [["Error during RPC call"]]
            callback["site"] = site
            if "Broker" in result and len(result["Broker"]) > 0:
                broker = []
                for i in result["Broker"]:
                    broker.append([str(i)])
            else:
                broker = [["Nothing to display"]]
            callback["broker"] = broker
            if "Owner" in result and len(result["Owner"]) > 0:
                owner = []
                for i in result["Owner"]:
                    owner.append([str(i)])
            else:
                owner = [["Nothing to display"]]
            callback["owner"] = owner

        self.finish(callback)

    ################################################################################
    def __request(self):
        self.numberOfJobs = int(self.get_argument("limit", "25"))
        self.pageNumber = int(self.get_argument("start", "0"))
        self.globalSort = [["SubmissionTime", "DESC"]]
        req = {}

        site = list(json.loads(self.get_argument("site", "[]")))
        if site:
            req["GridSite"] = site

        taskQueueId = list(json.loads(self.get_argument("taskQueueId", "[]")))
        if taskQueueId:
            req["TaskQueueID"] = taskQueueId

        pilotId = list(json.loads(self.get_argument("pilotId", "[]")))
        if pilotId:
            req["PilotJobReference"] = pilotId

        broker = list(json.loads(self.get_argument("broker", "[]")))
        if broker:
            req["broker"] = broker

        status = list(json.loads(self.get_argument("status", "[]")))
        if status:
            req["Status"] = status

        ce = list(json.loads(self.get_argument("computingElement", "[]")))
        if ce:
            req["DestinationSite"] = ce

        owner = list(json.loads(self.get_argument("owner", "[]")))
        if owner:
            req["Owner"] = owner

        ownerGroup = list(json.loads(self.get_argument("ownerGroup", "[]")))
        if ownerGroup:
            req["OwnerGroup"] = ownerGroup

        sort = json.loads(self.get_argument("sort", "[]"))
        if len(sort) > 0:
            self.globalSort = [[i["property"], i["direction"]] for i in sort]

        if self.get_argument("startDate", ""):
            req["FromDate"] = self.get_argument("startDate")
            if self.get_argument("startTime", ""):
                req["FromDate"] += " " + self.get_argument("startTime")

        if self.get_argument("endDate", ""):
            req["ToDate"] = self.get_argument("endDate")
            if self.get_argument("endTime", ""):
                req["ToDate"] += " " + self.get_argument("endTime")

        if self.get_argument("date", ""):
            req["LastUpdate"] = self.get_argument("date")
        gLogger.info("REQUEST:", req)
        return req

    @asyncGen
    def web_getJobInfoData(self):
        callback = {}
        data = self.get_argument("data")

        RPC = PilotManagerClient()
        if self.get_argument("data_kind") == "getPilotOutput":
            result = yield self.threadTask(RPC.getPilotOutput, data)
            if result["OK"]:
                callback = {"success": "true", "result": result["Value"]["StdOut"]}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getPilotError":
            result = yield self.threadTask(RPC.getPilotOutput, data)
            if result["OK"]:
                if len(result["Value"]["StdErr"]) > 0:
                    callback = {"success": "true", "result": result["Value"]["StdErr"]}
                else:
                    callback = {"success": "false", "error": "Pilot Error is empty"}
            else:
                callback = {"success": "false", "error": result["Message"]}
        elif self.get_argument("data_kind") == "getLoggingInfo":
            result = yield self.threadTask(RPC.getPilotLoggingInfo, data)
            if result["OK"]:
                callback = {"success": "true", "result": result["Value"]}
            else:
                callback = {"success": "false", "error": result["Message"]}

        self.finish(callback)

    @asyncGen
    def web_getStatisticsData(self):
        req = self.__request()

        paletteColor = Palette()

        RPC = PilotManagerClient()

        selector = self.get_argument("statsField")

        if selector == "Site":
            selector = "GridSite"
        if selector == "Computing Element":
            selector = "DestinationSite"
        if selector == "Owner Group":
            selector = "OwnerGroup"
        if selector == "Owner":
            selector = "OwnerDN"

        result = yield self.threadTask(RPC.getPilotStatistics, selector, req)
        if not result["OK"]:
            if "FromDate" in req:
                del req["FromDate"]

            if "LastUpdate" in req:
                del req["LastUpdate"]

            if "ToDate" in req:
                del req["ToDate"]

            result = yield self.threadTask(RPC.getCounters, "PilotAgents", [selector], req)

            statistics = {}
            if result["OK"]:
                for status, count in result["Value"]:
                    if "OwnerDN" in status:
                        userName = getUsernameForDN(status["OwnerDN"])
                        if userName["OK"]:
                            status["OwnerDN"] = userName["Value"]
                    statistics[status[selector]] = count

            result = S_OK(statistics)

        if result["OK"]:
            callback = []
            result = dict(result["Value"])
            keylist = sorted(result)
            if selector == "Site":
                tier1 = gConfig.getValue("/WebApp/PreferredSites", [])
                if len(tier1) > 0:
                    tier1.sort()
                    for i in tier1:
                        if i in result:
                            countryCode = i.rsplit(".", 1)[1]
                            callback.append(
                                {
                                    "key": i,
                                    "value": result[i],
                                    "code": countryCode,
                                    "color": paletteColor.getColor(countryCode),
                                }
                            )
            for key in keylist:
                if selector == "Site" and tier1:
                    if key not in tier1:
                        try:
                            countryCode = key.rsplit(".", 1)[1]
                        except BaseException:
                            countryCode = "Unknown"
                        callback.append(
                            {"key": key, "value": result[key], "code": countryCode, "color": paletteColor.getColor(key)}
                        )
                elif selector == "Site" and not tier1:
                    try:
                        countryCode = key.rsplit(".", 1)[1]
                    except BaseException:
                        countryCode = "Unknown"
                    callback.append(
                        {"key": key, "value": result[key], "code": countryCode, "color": paletteColor.getColor(key)}
                    )
                else:
                    callback.append({"key": key, "value": result[key], "code": "", "color": paletteColor.getColor(key)})
            callback = {"success": "true", "result": callback}
        else:
            callback = {"success": "false", "error": result["Message"]}

        self.finish(callback)



import os
from io import open
from tornado.template import BaseLoader, Template


class TemplateLoader(BaseLoader):
    def __init__(self, pathList, **kwargs):
        super(TemplateLoader, self).__init__(**kwargs)
        self.pathList = pathList

    def resolve_path(self, name, parent_path=None):
        if parent_path and not parent_path.startswith("<"):
            if not parent_path.startswith("/") and not name.startswith("/"):
                name = os.path.join(os.path.dirname(parent_path), name)
        return name

    def _create_template(self, name):
        for path in self.pathList:
            try:
                f = open(os.path.abspath(os.path.join(path, name)), "rb")
            except IOError:
                continue
            template = Template(f.read(), name=name, loader=self)
            f.close()
            return template
        raise RuntimeError("Can't find template %s" % name)
